/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/access.c,v 1.7 1996/03/02 00:37:58 mcooper Exp $";
#endif	/* lint */

/*
 * Access functions
 */

#include "defs.h"

/*
 * Free access list
 */
void
ACCESSfree(AccessInfo)
    AccessInfo_t	       *AccessInfo;
{
    register KeyVal_t	       *Ptr, *Last;

    for (Ptr = AccessInfo->List; Ptr; ) {
	Last = Ptr->Next;
	Ptr->Next = NULL;
	if (Ptr->Key)
	    (void) free(Ptr->Key);
	if (Ptr->Value)
	    (void) free(Ptr->Value);
	Ptr = Last;
	if (Last)
	    (void) free(Last);
    }
    AccessInfo->List = NULL;

    if (AccessInfo->Name) {
	(void) free(AccessInfo->Name);
	AccessInfo->Name = NULL;
    }
}

/*
 * Set access list (Variable) using string StrList.
 */
int
ACCESSset(Name, StrList, Variable, ErrBuff)
    char		       *Name;
    char		       *StrList;
    Opaque		        Variable;
    char		       *ErrBuff;
{
    AccessInfo_t	       *AccessInfo;
    AccessInfo_t	       *AccessPtr;
    register KeyVal_t	       *KeyVal;
    register KeyVal_t	       *KeyPtr;
    static char		        ErrBuf[BUFSIZ];
    char		       *Key;
    char		       *Value;
    static char		      **Argv = NULL;
    static int			Argc = 0;
    register int		i;

    if (Argc && Argv)
	DestroyArgv(&Argv, Argc);

    if (!Variable) {
	if (ErrBuff) sprintf(ErrBuff, "NULL Variable specified");
	return(-1);
    }

    AccessInfo = (AccessInfo_t *) Variable;
    AccessInfo->Name = strdup(Name);

    /*
     * StrList can look like:
     *
     *		net=<address>, host=<name|address>, ...
     */
    Argc = StrToArgv(StrList, ",", &Argv, SF_RMWS);
    for (i = 0; i < Argc; ++i) {
	if (!Argv[i] || !Argv[i][0])
	    continue;
	Key = GetKey(Argv[i], ErrBuf);
	if (!Key) {
	    if (ErrBuff)
		sprintf(ErrBuff, "Cannot parse key from `%s': %s.", 
			StrList, ErrBuf);
	    return(-1);
	}
	if (!EQC(Key, KW_NET) && !EQC(Key, KW_HOST)) {
	    if (ErrBuff)
		sprintf(ErrBuff, "Invalid access keyword `%s'.", Key);
	    return(-1);
	}
	Value = GetValue(Argv[i], ErrBuf);
	KeyVal = (KeyVal_t *) xcalloc(1, sizeof(KeyVal_t));
	if (Key)
	    KeyVal->Key = strdup(Key);
	if (Value)
	    KeyVal->Value = strdup(Value);
	if (AccessInfo->List) {
	    for (KeyPtr = AccessInfo->List; KeyPtr && KeyPtr->Next; 
		 KeyPtr = KeyPtr->Next);
	    KeyPtr->Next = KeyVal;
	} else
	    AccessInfo->List = KeyVal;
    }

    DestroyArgv(&Argv, Argc);

    return(0);
}
