/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/age.c,v 1.28 1996/03/02 00:38:10 mcooper Exp $";
#endif	/* lint */

/*
 * Age functions
 */

#include "defs.h"

/*
 * Initialize a AGEinfo_t.
 * All strings must be non-NULL for XDR.
 */
void
AGEinit(InfoPtr)
    AGEinfo_t		       *InfoPtr;
{
    (void) memset((char *)InfoPtr, 0, sizeof(AGEinfo_t));
    InfoPtr->UserName = "";
    ResultInit(&InfoPtr->Result);
}

/*
 * Convert an AGEinfo_t data type to a string.
 */
char *
AGEgetstr(Info)
    AGEinfo_t		       *Info;
{
    static char			Buff[MAXDBVAL];

    if (!Info)
	return((char *) NULL);

    (void) sprintf(Buff, "%s:%d:%d:%d:%d:%d:%d:%d",
		   Info->UserName,
		   Info->ModTime,
		   Info->MinChg,
		   Info->MaxValid,
		   Info->WarnTime,
		   Info->Inactive,
		   Info->ExpireDate,
		   Info->Flags);

    return(Buff);
}

/*
 * Get key from AGE string.
 */
char *
AGEgetkey(String)
    char		       *String;
{
    return(DBgetkeyuser(String));
}

/*
 * Get name of AGE database file
 */
char *
AGEgetfile(DomainName, ErrBuff)
    char		       *DomainName;
    char		       *ErrBuff;
{
    static char		        File[MAXPATHLEN];
    static char		        Buff[BUFSIZ];
    char		       *Domain;
    char		       *Err = NULL;

    if (ErrBuff)
	Err = ErrBuff;
    else
	Err = Buff;

    Domain = DomainName ? DomainName : GetDomain(NULL);

    if (strlen(AgeDir) + strlen(Domain) + 2 >= sizeof(File)) {
	(void) sprintf(Err, "Size of \"%s\" + \"%.100s\" is too large.", 
		       AgeDir, Domain);
	if (!ErrBuff)
	    Error("%s", Err);
	return((char *)NULL);
    }

    (void) sprintf(File, "%s/%s", AgeDir, Domain);

    return(File);
}

/*
 * Convert a AGEinfo string into an AGEinfo_t data type.
 */
AGEinfo_t *
AGEgetinfo(String)
    char		       *String;
{
    static AGEinfo_t	       *Info = NULL;
    char	       	       *cp;
    char		       *StrPtr = String;

    if (!Info)
	Info = (AGEinfo_t *) xmalloc(sizeof(AGEinfo_t));
    else {
	if (Info->UserName)
	    (void) free(Info->UserName);
    }
    memset((char *)Info, 0, sizeof(AGEinfo_t));	/* Zap Info */

    if (cp = strchr(StrPtr, ':'))
	*cp = CNULL;
    Info->UserName = strdup(StrPtr);

    if (!cp)
	return(Info);

    StrPtr = ++cp;
    Info->ModTime = (time_t) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    StrPtr = ++cp;
    Info->MinChg = (int) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    StrPtr = ++cp;
    Info->MaxValid = (int) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    StrPtr = ++cp;
    Info->WarnTime = (int) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    StrPtr = ++cp;
    Info->Inactive = (int) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    StrPtr = ++cp;
    Info->ExpireDate = (time_t) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    StrPtr = ++cp;
    Info->Flags = (int) strtol(StrPtr, &cp, 0);
    if (cp == StrPtr)
	return(Info);

    return(Info);
}

/*
 * Get age entry by matching UserName from database DBinfo.
 */
AGEinfo_t *
_AGEgetbyuser(DBinfo, UserName)
    DBinfo_t		       *DBinfo;
    char		       *UserName;
{
    static AGEinfo_t	        Info;
    static DBarg_t	        DBkey;
    static DBarg_t	        DBresult;
    AGEinfo_t		       *AGEptr;
    char		       *StrPtr;
    int				Status;

    errno = 0;
    if (!DBinfo)
	return((AGEinfo_t *) NULL);

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);
    DBresult.Data = (void *) NULL;
    DBresult.Size = (size_t) 0;

    AGEinit(&Info);

    Status = DBget(DBinfo, &DBkey, &DBresult);

    if (Status == 0) {			/* Success */
	StrPtr = (char *) DBresult.Data;
	StrPtr[DBresult.Size] = CNULL;
	if (AGEptr = AGEgetinfo(StrPtr)) {
	    AGEptr->Result.Status = RS_OK;
	    AGEptr->Result.Message = "";/* Required for XDR */
	    return(AGEptr);
	}
    } else if (Status == 1) {		/* No such entry */
	Info.Result.Status = RS_NOENT;
    } else {				/* Error occured */
	Info.Result.Message = SYSERR;
    }

    return(&Info);
}

/*
 * Get age entry by matching UserName.
 */
AGEinfo_t *
AGEgetbyuser(UserName, Domain)
    char		       *UserName;
    char		       *Domain;
{
    char		       *File;
    static AGEinfo_t		Info;
    AGEinfo_t		       *InfoPtr;
    DBinfo_t		       *DBinfo;
    static char		        Buff[BUFSIZ];

    /*
     * Prepare error result in case we need it.
     */
    AGEinit(&Info);

    File = AGEgetfile(Domain, Buff);
    if (!File) {
	Error("%s", Buff);
	Info.Result.Message = Buff;
	return(&Info);
    }

    if (!(DBinfo = DBopen(File, O_RDONLY))) {
	if (errno == ENOENT)
	    Info.Result.Status = RS_NOENT;
	(void) sprintf(Buff, "Cannot open %s: %s.", File, SYSERR);
	Error("%s", Buff);
	Info.Result.Message = Buff;
	return(&Info);
    }
	
    InfoPtr = _AGEgetbyuser(DBinfo, UserName);

    (void) DBclose(DBinfo);

    return (InfoPtr);
}

/*
 * Add age entry to database.
 */
char *
AGEadd(UserName, Domain, Age)
    char		       *UserName;
    char		       *Domain;
    AGEinfo_t		       *Age;
{
    static char			ErrBuff[BUFSIZ];
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    char		       *Status = NULL;

    ErrBuff[0] = CNULL;
    DBfile = AGEgetfile(Domain, ErrBuff);
    if (!DBfile)
	return(ErrBuff);

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT)))
	return(Format("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR));

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(DBkey.Data);
    DBdata.Data = (void *) AGEgetstr(Age);
    DBdata.Size = (size_t) strlen(DBdata.Data);

    if (DBstore(DBinfo, &DBkey, &DBdata, 0) != 0)
	Status = Format("Database write failed: %s", SYSERR);

    (void) DBclose(DBinfo);

    return(Status);
}

/*
 * Update an age entry after a password change
 */
char *
AGEupdate(UserName, Domain)
    char		       *UserName;
    char		       *Domain;
{
    static AGEinfo_t		Age;
    AGEinfo_t		       *AgePtr;
    char		       *cp;

    AgePtr = AGEgetbyuser(UserName, Domain);
    if (!AgePtr || AgePtr->Result.Status == RS_NOENT) {
	/*
	 * Entry didn't exist, so create a new one.
	 */
	(void) memset((char *)&Age, 0, sizeof(Age));
	Age.UserName = UserName;
	AgePtr = &Age;
    } else if (AgePtr->Result.Status != RS_OK)
	return(Format("Cannot get age data: %s", 
		      ResultErrStr(&AgePtr->Result)));

    (void) time(&AgePtr->ModTime);

    cp = AGEadd(UserName, Domain, AgePtr);
    if (cp)
	return(cp);

    /* Success */
    return((char *) NULL);
}

/*
 * Show AGE information about a given username.
 */
int
AGEshow(UserName, Age)
    char		       *UserName;
    AGEinfo_t		       *Age;
{
    if (!Age->UserName) {
	Error("No age information available for \"%s\".", UserName);
	return(-1);
    }

#define AS(s)	( (s) ? s : "(none)" )
    printf("Password age information for %s:\n", UserName);
    printf("%40s: %s\n", "Password Last Changed", 
	   AS(GetTimeStr(Age->ModTime)));
    printf("%40s: %s\n", "Password Expiration Date", 
	   AS(GetTimeStr(Age->ExpireDate)));
    printf("%40s: %d\n", "Minimum # of days between changes", 
	   Age->MinChg);
    printf("%40s: %d\n", "Maximum # of days password is valid", 
	   Age->MaxValid);
    printf("%40s: %d\n", "No. of warning days", Age->WarnTime);
    printf("%40s: %d\n", "No. of allowed inactive days", Age->Inactive);
#undef AS

    return(0);
}

/*
 * Read AGE information.
 */
int
AGEread(FileName, DomainName)
    char		       *FileName;
    char		       *DomainName;
{
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    int				Status;
    FILE		       *fp;

    if (!FileName || EQ(FileName, "-"))
	fp = stdout;
    else
	if (!(fp = fopen(FileName, "w+"))) {
	    Error("Cannot open \"%s\" for writing: %s.", FileName, SYSERR);
	    exit(1);
	}

    DBfile = AGEgetfile(DomainName, NULL);
    if (!DBfile) {
	Error("Cannot determine database file name.");
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDONLY))) {
	Error("Cannot open \"%s\" for reading: %s.", DBfile, SYSERR);
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    for (Status = DBiterate(DBinfo, &DBkey, &DBdata, TRUE); Status == 0; 
	 Status = DBiterate(DBinfo, &DBkey, &DBdata, FALSE)) {
	if (DoKeys)
	    (void) fprintf(fp, "%.*s ", DBkey.Size, DBkey.Data);
	(void) fprintf(fp, "%.*s\n", DBdata.Size, DBdata.Data);
    }

    if (Status == -1)
	Error("Database sequential fetch error: %s.", SYSERR);
    else if (Status == 1)	/* Normal "end of entries" indicator */
	Status = 0;

    (void) DBclose(DBinfo);
    if (fp != stdin)
	(void) fclose(fp);

    return(Status);
}

/*
 * Write AGE information
 */
int
AGEwrite(FileName, DomainName)
    char		       *FileName;
    char		       *DomainName;
{
    static char			Buff[MAXDBVAL];
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    static DBarg_t		DBkey;
    static DBarg_t		DBdata;
    register int		LineNo;
    register int		Len;
    int				Status = -1;
    FILE		       *fp;

    if (!FileName || EQ(FileName, "-"))
	fp = stdin;
    else
	if (!(fp = fopen(FileName, "r"))) {
	    Error("Cannot open \"%s\" for reading: %s.", FileName, SYSERR);
	    exit(1);
	}

    DBfile = AGEgetfile(DomainName, NULL);
    if (!DBfile) {
	Error("Cannot determine database file name.");
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	if (fp != stdout)
	    (void) fclose(fp);
	return(-1);
    }

    LineNo = 0;
    while (fgets(Buff, sizeof(Buff), fp)) {
	++LineNo;
	Len = strlen(Buff);
	if (Buff[Len - 1] == '\n')
	    Buff[--Len] = CNULL;

	if (!(DBkey.Data = (void *) AGEgetkey(Buff))) {
	    Error("Error: line %d: Cannot parse key.", LineNo);
	    Status = 1;
	    continue;
	}
	DBkey.Size = (size_t) strlen(DBkey.Data);
	DBdata.Data = (void *) Buff;
	DBdata.Size = (size_t) Len;

	if ( DBstore(DBinfo, &DBkey, &DBdata, 0) != 0) {
	    Error("Error: line %d: Database write failed: %s.", 
		  LineNo, SYSERR);
	    Status = 1;
	}
    }

    (void) DBclose(DBinfo);
    if (fp != stdin)
	(void) fclose(fp);

    return(Status);
}

/*
 * Set an Age argument.
 *
 * SetArg looks like "key1=value1,key2=value2,....".
 */
int
AGEset(UserName, Domain, SetArg)
    char		       *UserName;
    char		       *Domain;
    char		       *SetArg;
{
    AGEinfo_t		       *Info;
    static char			ErrBuff[BUFSIZ];
    char		       *ErrStr = NULL;
    char		       *Entry;
    char		       *Key;
    char		       *Value;

    Info = AGEgetbyuser(UserName, Domain);
    if (!Info || Info->Result.Status != RS_OK) {
	if (Debug) Message(
	   "WARNING: No current age entry for %s@%s: Creating new entry.",
			   UserName, Domain);
	Info = (AGEinfo_t *) xmalloc(sizeof(AGEinfo_t));
	AGEinit(Info);
	Info->UserName = UserName;
    }

    for (Entry = strtok(SetArg, ","); Entry; Entry = strtok(NULL, ",")) {
	Key = GetKey(Entry, ErrBuff);
	if (!Key) {
	    Error("Error parsing key from `%s': %s.", Entry, ErrBuff);
	    exit(1);
	}
	Value = GetValue(Entry, ErrBuff);
	/* A NULL Value is Okay */
#define NGET(s)	( (s) ? (int) strtol(s, NULL, 0) : 0 )
	if (EQC(Key, KW_MINCHG))
	    Info->MinChg = NGET(Value);
	else if (EQC(Key, KW_MAXVALID))
	    Info->MaxValid = NGET(Value);
	else if (EQC(Key, KW_WARNTIME))
	    Info->WarnTime = NGET(Value);
	else if (EQC(Key, KW_INACTIVE))
	    Info->Inactive = NGET(Value);
#undef NGET
	else if (EQC(Key, KW_EXPIREDATE)) {
	    if (Value && 
		(Info->ExpireDate = TIMEparse(Value, ErrBuff))==(time_t)-1) {
		Error("Error parsing expire date from `%s': %s.",
		      Entry, ErrBuff);
		exit(1);
	    }
	} else {
	    Error("Invalid keyword `%s'.", Key);
	    exit(1);
	}
    }

    (void) time(&(Info->ModTime));
    ErrStr = AGEadd(UserName, Domain, Info);
    if (ErrStr) {
	Error("Update of age entry for %s@%s failed: %s.",
	      UserName, Domain, ErrStr);
	return(-1);
    }

    /* Success! */
    return(0);
}

/*
 * Delete age entry from database.
 */
int
AGEdelete(UserName, Domain)
    char		       *Domain;
    char		       *UserName;
{
    static char			ErrBuff[BUFSIZ];
    static DBarg_t		DBkey;
    char		       *DBfile;
    DBinfo_t		       *DBinfo;
    int				Status = 0;

    ErrBuff[0] = CNULL;
    DBfile = AGEgetfile(Domain, ErrBuff);
    if (!DBfile) {
	Error("%s", ErrBuff);
	return(-1);
    }

    if (!(DBinfo = DBopen(DBfile, O_RDWR|O_CREAT))) {
	Error("Cannot open \"%s\" for writing: %s.", DBfile, SYSERR);
	return(-1);
    }

    DBkey.Data = (void *) UserName;
    DBkey.Size = (size_t) strlen(UserName);

    if (DBdelete(DBinfo, &DBkey) != 0) {
	Error("Database delete failed: %s", SYSERR);
	Status = -1;
    }

    (void) DBclose(DBinfo);

    return(Status);
}
