/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/ask.c,v 1.26 1996/03/02 00:38:25 mcooper Exp $";
#endif	/* lint */

/*
 * Ask user for information
 */

#include "defs.h"

struct passwd		       *Pwd;

/*
 * Ask user for a password.
 */
int
ASKpassword(Prompt, PassBuff, PassSize)
    char		       *Prompt;
    char		       *PassBuff;
    size_t			PassSize;
{
    register char	       *cp;
    register int		Len;

    cp = getpass(Prompt);
    if (cp && *cp) {
	Len = strlen(cp);
	if (Len < PassSize)
	    (void) strcpy(PassBuff, cp);
	else {
	    (void) strncpy(PassBuff, cp, PassSize);
	    PassBuff[PassSize - 1] = CNULL;
	}
    } else
	return(-1);

    return(0);
}

/*
 * Ask user for their new password.
 */
char *
ASKnewpwd(Query, Facility, ArgStr, ErrBuff)
    SPMquery_t		       *Query;
    Facility_t		       *Facility;
    char		       *ArgStr;
    char		       *ErrBuff;
    /*ARGSUSED*/
{
    register char	       *cp;
    char		       *EncPass;
    static char			Pass[BUFSIZ];
    DBstatus_t			Status;

    if (ASKpassword("New Password:", Pass, sizeof(Pass)) < 0)
	return((char *) NULL);

    /*
     * See if this is a safe password to use.
     */
    Pwd = getpwnam(Query->UserName);
    if (!Pwd) {
	Error("Cannot find password entry for %s.", Query->UserName);
	return((char *) NULL);
    }
    cp = (char *) safepwd(Pwd, Pass);
    if (cp) {
	Error("%s", cp);
	return((char *) NULL);
    }

    /*
     * Check to see if this password has been used before.
     */
    Status = DBcheckpwd(Query, Pass);
    if (Status == DB_TRUE) {
	Error("Please use a password that you have not used before.");
	return((char *) NULL);
    } 
#ifdef notdef
    /* We'll be kind for now */
    else if (Status == DB_ERROR) {
	Error("Cannot obtain your password history.");
	return((char *) NULL);
    }
#endif	/* notdef */

    /*
     * Verify password
     */
    cp = getpass("Retype New Password:");
    if (!EQ(cp, Pass)) {
	Error("Password mismatch - password left unchanged.");
	return((char *) NULL);
    }

    EncPass = getcrypt(Pass);
    if (EncPass) {
	cp = PWDvalidfield(EncPass, PC_PWD);
	if (cp) {
	    Error("%s", cp);
	    return((char *) NULL);
	}
	return(EncPass);
    }

    return((char *) NULL);
}

/*
 * Set signals for ASK*().
 */
void
ASKsignalSet(What)
    void		      (*What)();
{
    signal(SIGINT, What);
    signal(SIGHUP, What);
    signal(SIGQUIT, What);
    signal(SIGBUS, What);
    signal(SIGIOT, What);
}

/*
 * Signal handler for ASK*() functions.
 */
void
ASKsigHandler(Sig)
    int				Sig;
    /*ARGUSED*/
{
    printf("\nInterupt received - Exiting . . .\n");
    TTYechoOn();
    exit(1);
}

/*
 * Prompt and ask user for input.
 * Place result in Buff.
 */
int
ASKprompt(Desc, PromptSym, Example, Default, Buff, BuffSize, Flags)
    char		       *Desc;
    char		       *PromptSym;
    char		       *Example;
    char		       *Default;
    char		       *Buff;
    size_t			BuffSize;
    long			Flags;
{
    static char			Prompt[BUFSIZ];
    register char	       *cp;
    register int		Len;
    register int		DefLen;

    (void) strcpy(Prompt, Desc);
    if (Example && *Example) {
	(void) strcat(Prompt, " (e.g. `");
	(void) strcat(Prompt, Example);
	(void) strcat(Prompt, "')");
    }
    if (Default && *Default) {
	(void) strcat(Prompt, " [");
	(void) strcat(Prompt, Default);
	(void) strcat(Prompt, "]");
    }
    (void) strcat(Prompt, PromptSym);

    (void) printf("%s", Prompt);
    (void) fflush(stdout);

    /*
     * Set signal handling and turn off key stroke echo'ing (if needed)
     */
    ASKsignalSet(ASKsigHandler);
    if (IS_ON(Flags, PDF_NOECHO))
	TTYechoOff();

    /*
     * Now read the input from the user
     */
    (void) fflush(stdin);
    if (fgets(Buff, BuffSize, stdin) == NULL)
	return(-1);

    /*
     * Reset tty: Re-enable echo'ing and set signal defaults.
     */
    if (IS_ON(Flags, PDF_NOECHO)) {
	printf("\n");
	TTYechoOn();
    }
    ASKsignalSet(SIG_DFL);

    cp = strchr(Buff, '\n');
    if (cp)
	*cp = CNULL;

    if (Buff[0] == CNULL)
	if (Default) {
	    DefLen = strlen(Default);
	    if (DefLen < BuffSize)
		Len = DefLen;
	    else
		Len = BuffSize - 1;
	    (void) strncpy(Buff, Default, Len);
	    Buff[Len] = CNULL;
	} else
	    return(-1);

    if (EQ(Buff, KW_NONE))
	Buff[0] = CNULL;

    return(0);
}

/*
 * Check answer.
 */
int ASKanswer(Answer, Wanted)
    char		       *Answer;
    char		       *Wanted;
{
    if (EQC(Answer, Wanted))
	return(1);

    /*
     * Check first letter
     */
    if (isupper(*Wanted) && islower(*Answer))
	*Answer = toupper(*Answer);
    if (*Answer == *Wanted)
	return(1);

    return(0);
}

/*
 * Ask user for their new GECOS
 */
char *
ASKgecos(Query, Facility, ArgStr, ErrBuff)
    SPMquery_t		       *Query;
    Facility_t		       *Facility;
    char		       *ArgStr;
    char		       *ErrBuff;
    /*ARGSUSED*/
{
    static char		       *Buffer = NULL;
    static char			Ans[10];
    struct passwd	       *Passwd;
    Gecos_t		       *GecosPtr;
    char		       *Msg;
    int				GecosLen = 0;
    int				SepLen = 0;

    Passwd = (*Facility->Getpwnam)(Query->UserName, Facility);
    PWDgetgecos(Passwd->pw_gecos, GecosList);
    if (GecosSep)
	SepLen = strlen(GecosSep);

    Message("");
    Message("Default values are printed inside of '[]'.");
    Message("To accept the default, press the <RETURN> key.");
    Message("To specify a blank entry, type the word `%s'.", KW_NONE);
    Message("");

#define DEF(s)	((s && *(s)) ? s : KW_NONE)
    for (GecosPtr = GecosList; GecosPtr; GecosPtr = GecosPtr->Next) {
	if (GecosPtr->MaxSize <= 0)
	    continue;

	GecosPtr->NewVal = (char *) calloc(1, GecosPtr->MaxSize);

	if (ASKprompt(GecosPtr->Label, ": ", GecosPtr->Example,
		      DEF(GecosPtr->OldVal),
		      GecosPtr->NewVal, (size_t)GecosPtr->MaxSize) < 0)
	return((char *) NULL);
    }
#undef DEF

    Message("");
    for (GecosPtr = GecosList; GecosPtr; GecosPtr = GecosPtr->Next) {
	if (!GecosPtr->NewVal)
	    continue;
	GecosLen += strlen(GecosPtr->NewVal) + SepLen;
	Message("%20s: %s", GecosPtr->Label, GecosPtr->NewVal);
    }
    Message("");
    if (ASKprompt("Is this information correct", "? ", NULL, KW_YES,
		  Ans, sizeof(Ans)) < 0)
	return((char *) NULL);

    if (!ASKanswer(Ans, KW_YES)) {
	Message("No information changed.");
	return((char *) NULL);
    }

    if (Buffer)
	(void) free(Buffer);
    Buffer = (char *) xmalloc(GecosLen + 1);
    Buffer[0] = CNULL;
    for (GecosPtr = GecosList; GecosPtr; GecosPtr = GecosPtr->Next) {
	if (Buffer[0])
	    strcat(Buffer, GecosSep);
	strcat(Buffer, GecosPtr->NewVal);
    }

    Msg = PWDvalidfield(Buffer, PC_GECOS);
    if (Msg) {
	Message("%s", Msg);
	return((char *) NULL);
    }

    return(Buffer);
}

/*
 * Ask user for their new SHELL
 */
char *
ASKshell(Query, Facility, ArgStr, ErrBuff)
    SPMquery_t		       *Query;
    Facility_t		       *Facility;
    char		       *ArgStr;
    char		       *ErrBuff;
    /*ARGSUSED*/
{
    static char		      **ShellList = NULL;
    static int			ShellCnt = 0;
    static char			NewShell[BUFSIZ];
    static char			Ans[10];
    struct passwd	       *Passwd;
    char		       *Shell;
    register int		i;
    int				Okay;

    if (ShellList)
	DestroyArgv(&ShellList, ShellCnt);

    Passwd = (*Facility->Getpwnam)(Query->UserName, Facility);

    /*
     * Count the number of shells and search for the user's current
     * shell in the acceptable list.
     */
    setusershell();
    for (ShellCnt = 0, Okay = 0, Shell = getusershell(); Shell; 
	 ++ShellCnt, Shell = getusershell()) {
	if (Passwd->pw_shell && EQ(Shell, Passwd->pw_shell))
	    Okay = 1;
    }
    endusershell();

    /*
     * We didn't find the user's current shell, so he should not
     * be allowed to change his shell.
     */
    if (!Okay) {
	(void) sprintf(ErrBuff, 
"Your current login shell (%s) indicates\n\t\
you are not allowed to change your login shell to something else.", 
		       Passwd->pw_shell);
	return((char *) NULL);
    }

    ShellList = (char **) xcalloc(ShellCnt + 1, sizeof(char *));
    setusershell();
    for (i = 0, Shell = getusershell(); Shell && i < ShellCnt; 
	 Shell = getusershell())
	if (access(Shell, X_OK) == 0)
	    ShellList[i++] = strdup(Shell);
    endusershell();

    /*
     * Check command line shell argument
     */
    if (ArgStr) {
	for (i = 0, Okay = FALSE; !Okay && ShellList[i]; ++i)
	    if (ShellList[i] && EQ(ShellList[i], ArgStr))
		Okay = TRUE;
	if (Okay)
	    return(ArgStr);
	else {
	    (void) sprintf(ErrBuff,
			   "The shell `%s' is not valid on this system.", 
			   ArgStr);
	    return((char *) NULL);
	}
    }

    for (Okay = FALSE; !Okay; ) {
	Message("");
	Message("Here is a list of shells you may choose from:");
	for (i = 0; ShellList[i]; ++i)
	    if (ShellList[i])
		Message("\t%s", ShellList[i]);
	Message("");
	
	Message("Old Shell: %s", Passwd->pw_shell);
	if (ASKprompt("New shell", ": ", NULL, NULL, 
		      NewShell, sizeof(NewShell)) < 0) {
	    Message("Login shell left unchanged.");
	    return((char *) NULL);
	}

	for (i = 0; !Okay && ShellList[i]; ++i)
	    if (ShellList[i] && EQ(ShellList[i], NewShell))
		Okay = TRUE;
	if (!Okay)
	    Message("You must enter a shell from the provided list.");
    }

    Message("");
    Message("You have selected \"%s\" as your new shell.", NewShell);
    if (ASKprompt("Is this information correct", "? ", NULL, KW_YES,
		  Ans, sizeof(Ans)) < 0)
	return((char *) NULL);

    if (!ASKanswer(Ans, KW_YES)) {
	Message("No information changed.");
	return((char *) NULL);
    }

    return(NewShell);
}
