/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
/*
 * $Header: /src/common/cmd/spm/2.x/RCS/config.h,v 1.42 1996/03/02 00:40:33 mcooper Exp $
 */

#ifndef __my_config_h__
#define __my_config_h__

#include "config-os.h"

/*
 * Name of super user (i.e. "root")
 * If defined, then users can specify this user's
 * password in place of the real user's password during verification.
 */
#define SUPER_USER		"root"

/* 
 * Set transport type 
 */
#define TRANS_TYPE		TRANS_UDP

/*
 * Configuration files
 */
#define CONFIG_LIST	"/var/local/conf/spm.conf:/usr/lsd/conf/spm/spm.conf"
#define SPM_CONF		CONFIG_LIST	/* SPM config file */
#define SPMD_CONF		CONFIG_LIST	/* SPM Server config file */
#define SPMTRANS_CONF		CONFIG_LIST	/* SPM Trans config file */
#define SPMD_PID		"/etc/spmd.pid"	/* PID file for spmd */

/*
 * Directories
 */
#define TOP_DIR			"/var/spm"		/* Top directory */
#define OLD_DIR			"/var/pms"		/* Old name */

/*
 * Database directories
 */
#define AGE_DIR			"/var/spm/age"		/* AGE db directory */
#define PD_DIR			"/var/spm/person"	/* PD db directory */
#define PWH_DIR			"/var/spm/pwdhist"	/* PWH db directory */
#define TRANS_DIR		"/var/spm/transactions"	/* Transactions */

/*
 * NIS definetions
 */
#define NIS_DIR			"/var/yp"		/* NIS directory */
#define NIS_PWDFILE		"/var/yp/src/passwd"	/* Pwd file */
#define NIS_LOCKFILE		"/var/yp/ptmp"		/* Pwd Lock file */
#define NIS_POSTCMD	\
	"PATH=/bin:/usr/ccs/bin:/usr/ucb; export PATH; \
	(cd %D && make %B) > %D/log.%B 2>&1"		/* Post op command */

/*
 * Local (/etc) password files
 */
#ifndef LOCAL_PWDFILE
#define LOCAL_PWDFILE		"/etc/passwd"	/* Local pwd file */
#endif
#ifndef LOCAL_LOCKFILE
#define LOCAL_LOCKFILE		"/etc/ptmp"	/* Local pwd lock file */
#endif

/*
 * Verify information
 */
#define VERIFY_NUM		2

/*
 * Shadow password files
 */
#if	defined(HAVE_SPWD)
#ifndef SHADOW_PWDFILE
#define SHADOW_PWDFILE		"/etc/shadow"	/* Shadow pwd file */
#endif
#ifndef SHADOW_LOCKFILE
#define SHADOW_LOCKFILE		"/etc/stmp"	/* Shadow pwd lock file */
#endif
#else	/* !HAVE_SPWD */
#ifndef SHADOW_PWDFILE
#define SHADOW_PWDFILE		NULL
#endif
#ifndef SHADOW_LOCKFILE
#define SHADOW_LOCKFILE		NULL
#endif
#endif	/* HAVE_SPWD */

#define GECOS_SEP		","		/* GECOS field seperator */
#define PD_SSN_FILE		"SSN"		/* SSN-only PD data file */
#define DOMAIN_LOCAL		"local"		/* Local domain name */
#define DOMAIN_DEFAULT		DOMAIN_LOCAL	/* Default domain name */
#define FILE_MODE		0600		/* Default file mode */
#define DIR_MODE		0700		/* Default dir mode */
#define UMASK_MODE		022		/* Default umask */
#define MAXDBVAL		900		/* Maximum size of DB value */
#define MAXDBKEY		100		/* Maximum size of DB key */

#define MAX_PWH_ENT		20		/* Max no. of pwd's kept */

#define MAX_ASK			5		/* Max times to ask user */

/*
 * RPC variables
 */
#define RPC_PING_TIMEOUT	10		/* RPC ping timeout (secs) */
#define RPC_CALL_TIMEOUT	120		/* RPC call timeout (secs) */
#define RPC_RETRY_TIMEOUT	30		/* RPC retry timeout (secs) */

/*
 * SPM Error parameters
 */
#define SPM_MAX_RETRIES		30		/* Max # of retries for SPM
						   errors */
#define SPM_RETRY_INTR		10		/* Sleep this amount in between
						   SPM error retries */

/*
 * Transaction definetions
 */
#define TRANS_MAX_CREATE_TRIES	30000		/* Max creat trans file tries*/
#define TRANS_PREFIX		"spmtr1"	/* Prefix for trans files */
#define TRANS_FLD_SEP		":"		/* Transact field seperator */
#define TRANS_FLD_CNT		4		/* Number of trans fields */

/*
 * Maximum password field sizes
 */
#define MAX_FLD_PWD		20		/* pw_passwd */
#define MAX_FLD_SHELL		100		/* pw_shell */

#define MAX_LOCK_TRIES		30		/* Maximum # of lock tries */
#define LOCK_RETRY_INT		10		/* Seconds between retry */

/*
 * Current (highest) version of SPM protocol the client should connect to
 */
#define SPMVERS			SPMVERS_2

/*
 * Characters that are considered seperators and are striped when
 * PDF_STRIPSEP (StripSep in spm.conf) is enabled.
 */
#define STRIP_SEP_CHARS		"/-"

#endif	/* __my_config_h__ */
