/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/dbcommon.c,v 1.6 1996/03/02 00:40:46 mcooper Exp $";
#endif	/* lint */

/*
 * Common database functions.
 */

#include "defs.h"
#include <ndbm.h>

/*
 * Open a database.
 */
DBinfo_t *
DBopen(File, Flags)
    char		       *File;
    int				Flags;
{
    DBinfo_t		       *DBinfo;
    DBM			       *DBptr;

    if (!File)
	return((DBinfo_t *) NULL);

    errno = 0;
    DBptr = dbm_open(File, Flags, FILE_MODE);

    if (!DBptr)
	return((DBinfo_t *) NULL);

    DBinfo = (DBinfo_t *) xmalloc(sizeof(DBinfo_t));
    DBinfo->FileName = strdup(File);
    DBinfo->Flags = Flags;
    DBinfo->Info = (Opaque) DBptr;

    return(DBinfo);
}

/*
 * Close a database.
 */
DBstatus_t
DBclose(DBinfo)
    DBinfo_t		       *DBinfo;
{
    errno = 0;
    if (!DBinfo)
	return(DB_ERROR);

    if (DBinfo->FileName)
	(void) free(DBinfo->FileName);
    if (DBinfo->Info) 			/* Call DB close routine */
	dbm_close( (DBM *) DBinfo->Info );
    (void) free(DBinfo);

    return(DB_OK);
}

/*
 * Store a value in a DB.
 */
DBstatus_t
DBstore(DBinfo, Key, Value, Flags)
    DBinfo_t		       *DBinfo;
    DBarg_t		       *Key;
    DBarg_t		       *Value;
    int				Flags;
    /*ARGSUSED*/
{
    static datum		dbmkey;
    static datum		dbmvalue;
    DBM			       *db;
    int				Status;

    db = (DBM *) DBinfo->Info;

    dbmkey.dptr = Key->Data;
    dbmkey.dsize = Key->Size;
    dbmvalue.dptr = Value->Data;
    dbmvalue.dsize = Value->Size;

    Status = dbm_store(db, dbmkey, dbmvalue, DBM_REPLACE);
    if (Status != 0 || dbm_error(db)) {
	dbm_clearerr(db);
	return(DB_ERROR);
    }

    return(DB_OK);
}

/*
 * Delete entry from database.
 * Return DB_OK on success.
 * Return DB_ERROR on error.
 */
DBstatus_t
DBdelete(DBinfo, Key)
    DBinfo_t		       *DBinfo;
    DBarg_t		       *Key;
{
    static datum		dbmkey;
    DBM			       *db;

    db = (DBM *) DBinfo->Info;
    dbmkey.dptr = Key->Data;
    dbmkey.dsize = Key->Size;

    if (dbm_delete(db, dbmkey) < 0)
	return(DB_ERROR);

    if (dbm_error(db)) {
	dbm_clearerr(db);
	return(DB_ERROR);
    }

    return(DB_OK);
}

/*
 * Iterate over every entry in a DB on each call.
 * The flag IsFirst indicates wheather we should
 * start with the first record or continue where
 * we left off from the last call.
 * Return DB_OK on success.
 * Return DB_NOENT when no more entries.
 * Return DB_ERROR on error.
 */
DBstatus_t
DBiterate(DBinfo, Key, Value, IsFirst)
    DBinfo_t		       *DBinfo;
    DBarg_t		       *Key;
    DBarg_t		       *Value;
    int				IsFirst;
{
    datum			dbmkey;
    datum			dbmvalue;
    DBM			       *db;

    db = (DBM *) DBinfo->Info;

    if (IsFirst)
	dbmkey = dbm_firstkey(db);
    else
	dbmkey = dbm_nextkey(db);
    if (dbm_error(db)) {
	dbm_clearerr(db);
	return(DB_ERROR);
    }
    if (!dbmkey.dptr)
	/* No more entries */
	return(DB_NOENT);

    if (Key->Data) {
	(void) free(Key->Data);
	Key->Data = NULL;
    }
    Key->Data = (void *) xmalloc(dbmkey.dsize);
    memcpy(Key->Data, dbmkey.dptr, dbmkey.dsize);
    Key->Size = dbmkey.dsize;

    dbmvalue = dbm_fetch( db, dbmkey);
    if (dbm_error(db)) {
	dbm_clearerr(db);
	return(DB_ERROR);
    }
    if (!dbmvalue.dptr)
	/* No more entries */
	return(DB_NOENT);

    if (Value->Data) {
	(void) free(Value->Data);
	Value->Data = NULL;
    }
    Value->Data = (void *) xmalloc(dbmvalue.dsize);
    memcpy(Value->Data, dbmvalue.dptr, dbmvalue.dsize);
    Value->Size = dbmvalue.dsize;

    return(DB_OK);
}

/*
 * Get entry with key Key from a database and place
 * return value in Value.
 * Return DB_OK on success.
 * Return DB_NOENT when no such entry.
 * Return DB_ERROR on error.
 */
DBstatus_t
DBget(DBinfo, Key, Value)
    DBinfo_t		       *DBinfo;
    DBarg_t		       *Key;
    DBarg_t		       *Value;
{
    static datum		dbmkey;
    static datum		dbmvalue;
    DBM			       *db;

    db = (DBM *) DBinfo->Info;
    dbmkey.dptr = Key->Data;
    dbmkey.dsize = Key->Size;

    dbmvalue = dbm_fetch(db, dbmkey);

    if (dbm_error(db)) {
	dbm_clearerr(db);
	return(DB_ERROR);
    }

    if (!dbmvalue.dptr)
	/* No such entry */
	return(DB_NOENT);

    Value->Data = dbmvalue.dptr;
    Value->Size = dbmvalue.dsize;

    return(DB_OK);
}

/*
 * Get username key from String.
 */
char *
DBgetkeyuser(String)
    char		       *String;
{
    static char			Buff[MAXDBKEY];
    register char	       *cp;

    cp = strchr(String, ':');
    if (!cp)
	return((char *)NULL);

    if (cp - String > sizeof(Buff))
	return((char *)NULL);

    (void) strncpy(Buff, String, cp - String);
    Buff[cp - String] = CNULL;

    return(Buff);
}
