/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/fac.c,v 1.15 1996/03/02 00:41:17 mcooper Exp $";
#endif	/* lint */

/*
 * Password Management System Facilities
 */

#include "defs.h"
#if	PWLOOK_TYPE == PWLOOK_NSSWITCH
#include <nsswitch.h>
#endif	/* PWLOOK_NSSWITCH */

/*
 * Facilities we support.
 * The order of the list is important if no PWLOOK_TYPE is defined.
 * In this case, each facility is checked in order for a given username.
 */
Facility_t Facilities[] = {
#ifdef HAVE_LOCAL
    { FAC_LOCAL,		FN_LOCAL,	
	  "Local /etc flat files",
	  LOCAL_PWDFILE,	LOCAL_LOCKFILE,
	  NULL,
	  LOCALcheck,		PWDchange, 	
	  LOCALgetpwnam,	LOCALgetserver,		LOCALgetDomain,
#if	defined(HAVE_SPWD)
	  SHADOW_PWDFILE,	SHADOW_LOCKFILE,
	  SPWDgetShadPass,	SPWDupdSpwdStr
#endif	/* HAVE_SPWD */
    },
    { FAC_LOCAL,		FN_FILES,	
	  "Local /etc flat files",
	  LOCAL_PWDFILE,	LOCAL_LOCKFILE,
	  NULL,
	  LOCALcheck,		PWDchange, 	
	  LOCALgetpwnam,	LOCALgetserver,		LOCALgetDomain,
#if	defined(HAVE_SPWD)
	  SHADOW_PWDFILE,	SHADOW_LOCKFILE,
	  SPWDgetShadPass,	SPWDupdSpwdStr
#endif	/* HAVE_SPWD */
    },
#endif	/* HAVE_LOCAL */

#ifdef HAVE_USERPW
    { FAC_LOCAL,		FN_LOCAL,
	  "Local /etc files",
	  LOCAL_PWDFILE,	LOCAL_LOCKFILE,
	  NULL,
	  LOCALcheck,		USERPWchange,
	  LOCALgetpwnam,	LOCALgetserver,		LOCALgetDomain,
	  NULL,			NULL,
	  USERPWgetShadPass 
    },
#endif	/* HAVE_USERPW */

#ifdef HAVE_HPUXPW
    { FAC_LOCAL,		FN_LOCAL,
	  "Local /etc files",
	  LOCAL_PWDFILE,	LOCAL_LOCKFILE,
	  NULL,
	  LOCALcheck,		PWDchange,
	  LOCALgetpwnam,	LOCALgetserver,		LOCALgetDomain,
	  SHADOW_PWDFILE,	SHADOW_LOCKFILE,
	  HPUXPWgetShadPass,	HPUXPWupdShadStr
    },
#endif	/* HAVE_USERPW */

#ifdef HAVE_NIS
    { FAC_NIS,	 		FN_NIS,
	  "Network Information Service (YP)",
	  NIS_PWDFILE,		NIS_LOCKFILE,
	  NIS_POSTCMD,
	  NIScheck,		PWDchange,
	  NISgetpwnam,		NISgetserver,		NISgetDomain 
    },
#endif	/* HAVE_NIS */

#ifdef HAVE_NISPLUS
    { FAC_NISPLUS,		FN_NISPLUS,	
	  "Network Information Service Plus",
	  NULL,			NULL,
	  NULL,
	  NISPLUScheck,		NISPLUSchange,	
	  NISPLUSgetpwnam,	NISPLUSgetserver,	NISPLUSgetDomain 
    },
#endif	/* HAVE_NISPLUS */
    { FAC_UNKNOWN, NULL },
};

/*
 * Look up the facility type
 */
Facility_t *
FAClookup(Name, BeSilent)
    char		       *Name;
    int				BeSilent;
{
    register int		i;

    for (i = 0; Facilities[i].Name; ++i)
	if (EQ(Name, Facilities[i].Name))
	    return(&Facilities[i]);

    if (!BeSilent)
	Error("Unknown facility type: %s.", Name);

    return((Facility_t *) NULL);
}

/*
 * Figure out of facility type
 */
Facility_t *
FACgetfac(UserName)
    char		       *UserName;
{
    Facility_t		       *Facility = NULL;
#if	PWLOOK_TYPE == PWLOOK_NSSWITCH
    /*
     * Use /etc/nsswitch.conf
     */
    enum __nsw_parse_err	nswerr;
    struct __nsw_switchconfig  *config;
    struct __nsw_lookup	       *lookups;
    register int		nlookups;
    char		       *Name;

    /*
     * Read /etc/nsswitch.conf
     */
    if (!(config = __nsw_getconfig(__NSW_PASSWD_DB, &nswerr))) {
	Error("__nsw_getconfig failed: %s", SYSERR);
	return((Facility_t *) NULL);
    }

    /*
     * Iterate over lookup types.
     */
    for (lookups = config->lookups, nlookups = config->num_lookups; 
	 lookups && nlookups; 
	 lookups = lookups->next, --nlookups) {

	if (Debug) Message("Trying nsswitch \"%s\" type ... ",
			   lookups->service_name);
	/*
	 * Special handling for "compat"
	 */
	if (EQ(lookups->service_name, "compat")) {
	    Facility = FAClookup(FN_LOCAL, FALSE);
	    if (Facility && (*Facility->Check)(UserName, Facility) == 0)
		break;
	    Facility = FAClookup(FN_NIS, FALSE);
	    if (Facility && (*Facility->Check)(UserName, Facility) == 0)
		break;
	    Facility = NULL;
	    continue;
	}

	Facility = FAClookup(lookups->service_name, FALSE);
	if (Facility && (*Facility->Check)(UserName, Facility) == 0)
	    break;
	Facility = NULL;
    }
#endif	/* PWLOOK_NSSWITCH */

#if	PWLOOK_TYPE == PWLOOK_NONE
    register int		i;

    /*
     * Check each defined facility
     */
    for (i = 0; Facilities[i].Name; ++i) {
	Facility = &Facilities[i];
	if ( (*Facility->Check)(UserName, Facility) == 0)
	    break;
	Facility = NULL;
    }
#endif	/* PWLOOK_NONE */

    return(Facility);
}

/*
 * Get the facility entry that UserName appears in.
 * If Type is not FAC_UNKNOWN, then use it as the facility.
 * Otherwise, figure out what facility UserName appears in
 * and use that facility.
 */
Facility_t *
FACget(UserName, Type)
    char		       *UserName;
    FACtype_t			Type;
{
    register int		i;
    Facility_t		       *Facility = NULL;

    if (Type == FAC_UNKNOWN) {
	Facility = FACgetfac(UserName);
	if (!Facility)
	    Error("Cannot find username \"%s\" in any password facility.",
		  UserName);
    } else
	for (i = 0; Facilities[i].Name; ++i)
	    if (Facilities[i].Type == Type) {
		Facility = &Facilities[i];
		if ( (*Facility->Check)(UserName, Facility) == 0)
		    return(Facility);
		else
		    return((Facility_t *)NULL);
	    }

    return(Facility);
}

/*
 * Get the field associated with Type for UserName from Facility.
 */
char *
FACgetByType(UserName, Type, Facility)
    char		       *UserName;
    PCtype_t			Type;
    Facility_t		       *Facility;
    /*ARGSUSED*/
{
    FILE		       *FilePtr;
    static char			Buff[BUFSIZ];
    char		       *PwdFile;
    char		       *Value = NULL;
    ShadPass_t		       *ShadPass;
    struct passwd	       *Passwd;

    /*
     * If the request is for the password field and the facility
     * uses a shadow password file, call that routine directly.
     */
    if (Type == PC_PWD && Facility->ShadowFile && Facility->GetShadName) {
	ShadPass = (*Facility->GetShadName)(UserName, Facility);
	if (ShadPass)
	    return(ShadPass->CrPass);
	else {
	    Error("Cannot find shadow entry for \"%s\".", UserName);
	    return((char *) NULL);
	}
    }

    PwdFile = Facility->PwdFile;
    FilePtr = fopen(PwdFile, "r");
    if (!FilePtr) {
	Error("Cannot open \"%s\": %s", PwdFile, SYSERR);
	return((char *) NULL);
    }

    while (fgets(Buff, sizeof(Buff), FilePtr)) {
	if (!PWDmatch(UserName, Buff))
	    continue;

	Passwd = PWDstrtopwd(Buff);
	switch (Type) {
	    /*
	     * Take care to return an empty string if the pwd field 
	     * is empty.
	     */
	case PC_PWD:	Value = SV(Passwd->pw_passwd);		break;
	case PC_GECOS:	Value = SV(Passwd->pw_gecos);		break;
	case PC_SHELL:	Value = SV(Passwd->pw_shell);		break;
	default:
	    /* Should never reach here */
	    Error("Unknown password change request: %d", Type);
	    (void) fclose(FilePtr);
	    return((char *) NULL);
	}
	break;
    }

    (void) fclose(FilePtr);

    return(Value);
}

PCtype_t PCtypeList[] = { PC_PWD, PC_SHELL, PC_GECOS };

/*
 * Call a Facility Change function with proper processing.
 *
 * TransProc will be TRUE if we're being called by a transaction
 * processing function.  That flag indicates we really just want to
 * perform a transaction and not queue one.
 */
SPMstat_t
FACchange(Facility, TransactList, TransProc, MsgBuff)
    Facility_t		       *Facility;
    Transact_t		       *TransactList;
    int				TransProc;
    char		       *MsgBuff;
{
    static char			ErrBuff[BUFSIZ];
    register int		i;
    SPMstat_t			Status;
    int				RetVal = 0;

    if (!TransProc) {
	if (TRANSenabled(Facility->Name))
	    return TRANSadd(TransactList, Facility, MsgBuff);
	else
	    return (*Facility->Change)(TransactList, TransactList->Type,
				       Facility, MsgBuff);
    }

    /*
     * We need to call the Change function foreach possible change type
     * so that changes of the same type can be processed all at once.
     */
    for (i = 0; i < (sizeof(PCtypeList) / sizeof(PCtype_t)); ++i) {
	if (Debug) Message("Doing all changes for %s", 
			   GetPCname(PCtypeList[i]));
	ErrBuff[0] = CNULL;
	Status = (*Facility->Change)(TransactList, PCtypeList[i], 
				     Facility, ErrBuff);
	if (Debug && ErrBuff[0])
	    Message("%s", ErrBuff);
	else if (Status != RS_OK && ErrBuff[0]) {
	    Error("%s", ErrBuff);
	    ++RetVal;
	}
    }

    if (RetVal)
	return(RS_ERROR);
    else
	return(RS_OK);
}
