/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/nis.c,v 1.10 1996/03/02 00:41:55 mcooper Exp $";
#endif	/* lint */

/*
 * Password Management System Facilities for "nis"
 */

#include "defs.h"

#if	defined(HAVE_NIS)
#include <rpcsvc/ypclnt.h>

/*
 * Get our NIS domain name.
 */
char *
NISgetDomain()
{
    static char			Buff[BUFSIZ];

    if (getdomainname(Buff, sizeof(Buff)) == 0)
	return(Buff);
    else
	return((char *) NULL);
}

/*
 * Check to see if UserName appears in the "nis"
 * password facility.
 */
int
NIScheck(UserName, Facility)
    char		       *UserName;
    Facility_t		       *Facility;
    /*ARGSUSED*/
{
    char		       *Domain;
    char		       *YPbuf;
    int				YPlen;
    int				YPerr;

    if (!(Domain = NISgetDomain()))
	return(-1);

    YPerr = yp_match(Domain, "passwd.byname", UserName, strlen(UserName), 
		     &YPbuf, &YPlen);

    if (YPerr == 0)
	/* We found it */
	return(0);

    /*
     * Entry didn't exist.
     */
    if (YPerr != YPERR_KEY)
	Error("NIS lookup of \"%s\" failed: %s.", 
	      UserName, yperr_string(YPerr));

    return(-1);
}

/*
 * Check to see if UserName appears in the "nis"
 * password facility.
 */
struct passwd *
NISgetpwnam(UserName, Facility)
    char		       *UserName;
    Facility_t		       *Facility;
    /*ARGSUSED*/
{
    char		       *Domain;
    char		       *YPbuf;
    int				YPlen;
    int				YPerr;
    register struct passwd     *PwdPtr = NULL;
    static char		       *StrPtr = NULL;

    if (!(Domain = NISgetDomain()))
	return(PwdPtr);

    YPerr = yp_match(Domain, "passwd.byname", UserName, strlen(UserName), 
		     &YPbuf, &YPlen);

    if (YPerr != 0) {
	if (Debug) Message("NIS lookup of \"%s\" failed: %s.", 
			   UserName, yperr_string(YPerr));
	return(PwdPtr);
    }

    if (StrPtr)
	(void) free(StrPtr);
    StrPtr = xmalloc(YPlen + 1);
    (void) strcpy(StrPtr, YPbuf);
    return(PWDstrtopwd(StrPtr));
}

/*
 * Get name of our NIS master
 */
char *
NISgetserver(Domain)
    char		       *Domain;
{
    int				YPerr;
    char		       *Master = NULL;

    if (!Domain || !*Domain) {
	Error("No NIS domain name specified.");
	return((char *) NULL);
    }

    YPerr = yp_master(Domain, "passwd.byname", &Master);
    if (YPerr != 0) {
	Error("Unable to determine the name of our NIS master: %s.",
	      yperr_string(YPerr));
	return((char *) NULL);
    }

    return(Master);
}

#endif	/* HAVE_NIS */
