/*
 * $Header: /src/common/usc/include/RCS/osconfig.h,v 1.47 1996/03/01 21:18:03 mcooper Exp mcooper $
 */

#ifndef __OSCONFIG_H__
#define __OSCONFIG_H__

#include "osconfig-data.h"
#include <stdio.h>

/*
 * This host is POSIX SOURCE compliant.
 */
#if	defined(sunos4) || defined(sunos5) || defined(aix3) || defined(sgi) ||\
	defined(ultrix4) || defined(__hpux)
#define IS_POSIX_SOURCE
#endif

/*
 * this really belongs in a per-module basis - some #includes break if
 * both POSIX & BSD are defined
 *
 * Enable BSD compatibility.
 *
 * #if	defined(aix3)
 * #ifndef _BSD
 * #define _BSD 	43
 * #endif
 * #endif
 */

/*
 * Arg type
 */
#if	!defined(ARG_TYPE)
# if	((defined(sunos4) || defined(__hpux)) && !defined(__STDC__))
#  define 	ARG_TYPE		ARG_VARARGS
#  include 	<varargs.h>
# endif
#endif	/* ARG_TYPE */
#if	!defined(ARG_TYPE) && \
  ((defined(IS_POSIX_SOURCE) && defined(__STDC__)) || defined(aix3))
#define 	ARG_TYPE		ARG_STDARG
#include 	<stdarg.h>
#endif	/* IS_POSIX_SOURCE */

/*
 * Directory style
 */
#if	!defined(DIR_TYPE) && defined(IS_POSIX_SOURCE)
/*
 * POSIX directory routines
 */
#define 	DIR_TYPE		DIR_DIRENT
#include 	<sys/types.h>
#include 	<dirent.h>
typedef 	struct dirent		direntry_t;
#define 	D_NAMLEN(p)		(strlen((p)->d_name))
#endif	/* IS_POSIX_SOURCE */
#if	!defined(DIR_TYPE)
/*
 * Assume old style BSD "direct".
 */
#define 	DIR_TYPE		DIR_DIRECT
#include 	<sys/dir.h>
typedef 	struct direct		direntry_t;
#define 	D_NAMLEN(p)		((p)->d_namlen)
#endif	/* !DIR_TYPE */

/*
 * Type of Non-Blocking I/O
 */
#if	!defined(NBIO_TYPE) && defined(IS_POSIX_SOURCE)
#define 	NBIO_TYPE		NBIO_FCNTL
#include 	<sys/types.h>
#include 	<fcntl.h>
#endif	/* NBIO_TYPE */
#if	!defined(NBIO_TYPE) && defined(FIONBIO)
#define 	NBIO_TYPE		NBIO_FIONBIO
#endif	/* NBIO_TYPE */

/*
 * Type of wait calls
 */
#if	!defined(WAIT_TYPE) && defined(IS_POSIX_SOURCE)
#define 	WAIT_TYPE		WAIT_WAITPID
#include 	<sys/types.h>
#include 	<sys/wait.h>
typedef		int			waitarg_t;
#endif	/* WAIT_WAITPID */
#if	!defined(WAIT_TYPE)
/*
 * BSD defaults
 */
#define 	WAIT_TYPE		WAIT_WAIT
#include 	<sys/types.h>
#include 	<sys/wait.h>
typedef		union wait		waitarg_t;
#endif	/* !WAIT_TYPE */

/*
 * Stat filesystem.
 */
#if	!defined(STATFS_TYPE)
#if defined(sunos5)
#define 	STATFS_TYPE		STATFS_SYSV
#include 	<sys/statvfs.h>
typedef		struct statvfs		statfs_t;
#endif /* sunos5 */
#endif	/* STATFS_TYPE */
#if	!defined(STATFS_TYPE)
#define 	STATFS_TYPE		STATFS_BSD
#include	<sys/vfs.h>
typedef		struct statfs		statfs_t;
#endif	/* STATFS_TYPE */

/*
 * Terminal I/O
 */
#if	!defined(TERMIO_TYPE) && defined(IS_POSIX_SOURCE)
#define		TERMIO_TYPE		TERMIO_TERMIOS
#endif	/* TERMIO_TYPE */

/*
 * How to get system error string
 */
#if	!defined(SYSERR) && defined(IS_POSIX_SOURCE)
extern		int errno;
#define		SYSERR			strerror(errno)
#endif	/* SYSERR */
#if	!defined(SYSERR)
extern		int 			errno;
extern		int 			sys_nerr;
extern		char			*sys_errlist[];
#define		SYSERR \
((errno>0 && errno<sys_nerr) ? sys_errlist[errno] : "(unknown system error)")
#endif	/* SYSERR */

/*
 * Signal handler type
 */
#if	!defined(SIGHAND_T) && (defined(IS_POSIX_SOURCE) || defined(nextstep3))
#define		SIGHAND_T		void
#else
#define		SIGHAND_T		int
#endif

/*
 * Get hostname
 */
#if	!defined(HOSTNAME_TYPE) && defined(IS_POSIX_SOURCE) && !defined(sunos4)
/*
 * SunOS 4 doesn't return full hostnames via uname().
 */
#define	HOSTNAME_TYPE	HOSTNAME_UNAME
#endif
#if	!defined(HOSTNAME_TYPE)
/*
 * Use the BSD gethostname() by default
 */
#define HOSTNAME_TYPE	HOSTNAME_GETHOSTNAME
#endif

/*
 * General things
 */
#ifndef CNULL
#define CNULL		'\0'
#endif	/* CNULL */

#if	defined(IS_POSIX_SOURCE)
/*
 * Common include files
 */
#include <unistd.h>
#include <string.h>
#ifndef bzero
#define bzero(p,s)	memset(p, '\0', (size_t) s)
#endif	/* bzero */
#ifndef bcopy
#define bcopy(f,t,s)	memcpy(t, f, (size_t) s)
#endif	/* bcopy */
#ifndef bcmp
#define bcmp(a,b,l)	memcmp(a, b, (size_t) l)
#endif	/* bcmp */
#ifndef index
#define	index		strchr
#endif	/* index */
#ifndef rindex
#define rindex		strrchr
#endif	/* rindex */
#ifndef getwd
#define getwd(s)	(char *)getcwd(s, sizeof(s))
#endif	/* getwd */

/*
 * Define fseek() L_* values for non POSIX systems.
 */
#ifndef L_SET
#define L_SET		SEEK_SET
#endif	/* L_SET */
#ifndef L_INCR
#define L_INCR		SEEK_CUR
#endif	/* L_INCR */
#ifndef L_XTND
#define L_XTND		SEEK_END
#endif	/* L_XTND */

#else	/* !IS_POSIX_SOURCE */

/*
 * Common include files
 */
#include <strings.h>
#ifndef memset
#define memset(p,c,s)	bzero(p, (int) s)
#endif	/* memset */
#ifndef memcpy
#define memcpy(t,f,s)	bcopy(f, t, (int) s)
#endif	/* memcpy */
#ifndef memcmp
#define memcmp(a,b,l)	bcmp(a, b, (int) l)
#endif	/* memcmp */
#ifndef strchr
#define	strchr		index
#endif	/* strchr */
#ifndef strrchr
#define strrchr		rindex
#endif	/* strrchr */
#ifndef getcwd
#define getcwd(b,s)	(char *)getwd(b)
#endif	/* getcwd */

/*
 * Define SEEK_* for fseek().
 */
#include <sys/file.h>
#ifndef SEEK_SET
#define SEEK_SET	L_SET
#endif	/* SEEK_SET */
#ifndef SEEK_CUR
#define SEEK_CUR	L_INCR
#endif	/* SEEK_CUR */
#ifndef SEEK_END
#define SEEK_END	L_XTND
#endif	/* SEEK_END */

#endif	/* IS_POSIX_SOURCE */

#endif	/*  __OSCONFIG_H__ */
