/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/pwd.c,v 1.48 1996/03/02 00:42:24 mcooper Exp $";
#endif	/* lint */

/*
 * SPM Password functions
 */

#include "defs.h"

#include <sys/types.h>
#include <unistd.h>
#include <sys/resource.h>
#include <signal.h>

int				AmPrived = FALSE;	/* Priv'ed flag */

/*
 * Make a list of file descriptors for debugging purposes
 */
static void DescDump(ErrMsg)
    char		       *ErrMsg;
{
#if	defined(DEBUG)
    struct rlimit		rlimit;
    register int		Desc;
    int				Flags = 0;
    int				Owner = 0;

    if (getrlimit(RLIMIT_NOFILE, &rlimit) == -1) {
	Error("getrlimit(RLIMIT_NOFILE) failed: %s", SYSERR);
	return;
    }
    Message("DescDump: Starting file descriptor dump [#descs=%d]", 
	    rlimit.rlim_cur);
    Message("DescDump: Error = %s", ErrMsg);

    for (Desc = 2; Desc < rlimit.rlim_cur; ++Desc) {
	if (fcntl(Desc, F_GETFL, &Flags) == -1) {
	    Message("DescDump: Desc %d GETFL failed: %s", Desc, SYSERR);
	    continue;
	}
	Message("DescDump: Desc %d is OPEN - flags = 0x%x", Desc, Flags);
	if (fcntl(Desc, F_GETOWN, &Owner) == -1) 
	    Message("DescDump: Desc %d GETOWN failed: %s", Desc, SYSERR);
	else
	    Message("DescDump: Desc %d Owner = %d", Desc, Owner);
    }
    Message("DescDump: End of file descriptor dump");
#endif	/* DEBUG */
}

int
PWDmatch(UserName, Line)
    char		       *UserName;
    char		       *Line;
{
    register char	       *cp;
    int				Status = 0;

    cp = strchr(Line, ':');
    if (!cp)	/* Error! */
	return(0);
    *cp = CNULL;
    if (EQ(UserName, Line))
	Status = 1;
    *cp = ':';

    return(Status);
}

/*
 * Convert a line from a password file into a passwd struct.
 */
struct passwd *
PWDstrtopwd(String)
    char		       *String;
{
    static struct passwd	Pwd;
    static char		      **Argv = NULL;
    static int			Argc = 0;
    char		       *cp;

    if (Argc && Argv)
	DestroyArgv(&Argv, Argc);

    (void) memset((char *)&Pwd, 0, sizeof(Pwd));
    cp = strchr(String, '\n');
    if (cp)
	*cp = CNULL;

#define PWD_FLD_CNT	7

    Argc = StrToArgv(String, ":", &Argv, 0);
    if (Argc != PWD_FLD_CNT) {
	Error("PWDstrtopwd(%s) wrong field count (found %d, wanted %d)",
	      String, Argc, PWD_FLD_CNT);
	return((struct passwd *) NULL);
    }

    Pwd.pw_name			= Argv[0];
    Pwd.pw_passwd 		= Argv[1];
    if (Argv[2]) Pwd.pw_uid 	= atoi(Argv[2]);
    if (Argv[3]) Pwd.pw_gid 	= atoi(Argv[3]);
    Pwd.pw_gecos 		= Argv[4];
    Pwd.pw_dir 			= Argv[5];
    Pwd.pw_shell 		= Argv[6];

    return(&Pwd);
}

/*
 * Convert a passwd struct to a string suitable for
 * placement in a passwd file.
 */
char *
PWDpwdtostr(Pwd)
    struct passwd 	       *Pwd;
{
    static char			Buff[BUFSIZ];

    if (strlen(SV(Pwd->pw_name)) + 1 +
	strlen(SV(Pwd->pw_passwd)) + 1 +
	strlen(ltoa(Pwd->pw_uid)) + 1 +
	strlen(ltoa(Pwd->pw_gid)) + 1 +
	strlen(SV(Pwd->pw_gecos)) + 1 +
	strlen(SV(Pwd->pw_dir)) + 1 +
	strlen(SV(Pwd->pw_shell)) + 1 + 2 > sizeof(Buff)) {
	Error("Password string buffer too small for user %s", Pwd->pw_name);
	return((char *) NULL);
    }

    (void) sprintf(Buff, "%s:%s:%d:%d:%s:%s:%s\n",
		   SV(Pwd->pw_name),
		   SV(Pwd->pw_passwd),
		   Pwd->pw_uid,
		   Pwd->pw_gid,
		   SV(Pwd->pw_gecos),
		   SV(Pwd->pw_dir),
		   SV(Pwd->pw_shell));

    return(Buff);
}

/*
 * Get UserName's encrypted password field.
 */
char *
PWDgetpass(UserName, Facility)
    char		       *UserName;
    Facility_t		       *Facility;
    /*ARGSUSED*/
{
    ShadPass_t		       *Shadow;
    struct passwd	       *Passwd;

    if (!Facility) {
	Passwd = getpwnam(UserName);
	if (Passwd)
	    return(Passwd->pw_passwd);
	else
	    return((char *) NULL);
    }

    if (Facility->GetShadName) {
	Shadow = (*Facility->GetShadName)(UserName, Facility);
	if (Shadow)
	    return(Shadow->CrPass);
	else if (Debug) 
	    Message("No shadow entry found for `%s'; trying /etc/passwd.",
		    UserName);
	/*
	 * If we don't find a shadow entry, we fall through
	 * and do a normal Facility->getpwnam() lookup because on some
	 * OSs, it's not mandatory to have a shadow file.
	 */
    }

    Passwd = (*Facility->Getpwnam)(UserName, Facility);
    if (Passwd)
	return(Passwd->pw_passwd);

    return((char *) NULL);
}


/*
 * Verify a user's current password.
 * Return 0 if specified password matches current password.
 * Return > 0 if password does not match.
 * Return < 0 on error.
 */
int
PWDverify(UserName, Domain, CurPass, Facility)
    char		       *UserName;
    char		       *Domain;
    char		       *CurPass;
    Facility_t		       *Facility;
    /*ARGSUSED*/
{
    char		       *CrPwd = NULL;
    Facility_t		       *SuperFac;

    AmPrived = FALSE;

    CrPwd = PWDgetpass(UserName, Facility);
    if (!CrPwd)
	return(-1);

    if (EQ(crypt(CurPass, CrPwd), CrPwd))
	/* Passwords match! */
	return(0);
    else {
	/* Passwords do NOT match */
#if	defined(SUPER_USER)
	SuperFac = FACgetfac(SUPER_USER);
	if (!SuperFac) {
	    Error("Super user `%s' does not exist.", SUPER_USER);
	    return(1);
	}
	CrPwd = PWDgetpass(SUPER_USER, SuperFac);
	if (!CrPwd)
	    Error("No password entry for `%s'.", SUPER_USER);
	if (CrPwd && EQ(crypt(CurPass, CrPwd), CrPwd)) {
	    /* Password match for SUPER_USER */
	    AmPrived = TRUE;
	    return(0);
	}
#endif	/* SUPER_USER */
	return(1);
    }
}

char *
PWDgettmp(File)
    char		       *File;
{
    static char			TmpFile[MAXPATHLEN];
    char		       *Ext = ".tmpXXXXXX";

    if (strlen(File) + strlen(Ext) + 1 > sizeof(TmpFile)) {
	Error("Temporary file name buffer too small");
	return((char *) NULL);
    }

    (void) sprintf(TmpFile, "%s%s", File, Ext);

    return(mktemp(TmpFile));
}

/*
 * Is File a valid lock file?
 * Return 1 if it is valid.
 * Return 0 if it is not valid.
 * Return -1 on error.
 */
int
PWDvalidLock(File)
    char		       *File;
{
    static char			Buff[100];
    FILE		       *FilePtr;
    register char	       *cp;
    pid_t			ProcID;
    struct stat			StatBuf;

    if (lstat(File, &StatBuf) != 0) {
	Error("Cannot lstat %s: %s", File, SYSERR);
	return(-1);
    }

    /*
     * Assume that if the lock file is anything but
     * a regular file, then we didn't make it and
     * it therefor might not be in a compatible format.
     */
    if (!S_ISREG(StatBuf.st_mode)) {
	if (Debug) Error("Lock file %s is not a regular file.", File);
	return(-1);
    }

    if (StatBuf.st_size == 0) {
	if (Debug) Message("Lock file %s is empty.", File);
	return(-1);
    }

    FilePtr = fopen(File, "r");
    if (!FilePtr) {
	Error("Cannot open lock file %s: %s", File, SYSERR);
	return(-1);
    }
    if (fchmod(fileno(FilePtr), 0644) != 0) {
	Error("Cannot chmod file %s: %s", File, SYSERR);
	(void) fclose(FilePtr);
	(void) unlink(File);
	return(-1);
    }

    /*
     * Read process ID.
     */
    if (!fgets(Buff, sizeof(Buff), FilePtr)) {
	Error("Cannot read lock file %s: %s", File, SYSERR);
	(void) fclose(FilePtr);
	return(-1);
    }
    (void) fclose(FilePtr);
    cp = strchr(Buff, '\n');
    if (cp)
	*cp = CNULL;

    /*
     * See if we got something that looks like a PID.
     */
    if (!Buff[0] || !isdigit(Buff[0]))
	return(0);
    ProcID = atoi(Buff);
    if (ProcID <= 1)
	return(0);

    /*
     * Make sure the locking process isn't ourself.
     * If it is, we declare it invalid.
     */
    if (ProcID == getpid()) {
	Error("Lock file %s is locked by ME!", File);
	return(0);
    }

    /*
     * See if the PID exists
     */
    if (kill(ProcID, 0) == 0)
	return(1);
    else
	return(0);
}

/*
 * Create the password file lock.
 */
PWDlock_t *
PWDlock(PwdFile, LockFile, MsgBuff)
    char		       *PwdFile;
    char		       *LockFile;
    char		       *MsgBuff;
{
    static PWDlock_t		LockInfo;
    static char			Buff[100];
    size_t			BuffLen;
    int				Count = 0;

    (void) memset((char *)&LockInfo, 0, sizeof(LockInfo));
    LockInfo.FileDesc = -1;
    LockInfo.LockFile = LockFile;

    do {
	LockInfo.FileDesc = open(LockFile, O_WRONLY|O_CREAT|O_EXCL, 0644);
	if (LockInfo.FileDesc > 0)
	    /* Got lock */
	    break;

	if (errno == EEXIST) {
	    /* LockFile exists */
	    if (PWDvalidLock(LockFile) == 0) {
		Message("Invalid lock file %s; removing . . .", LockFile);
		if (unlink(LockFile) < 0) {
		    (void) sprintf(MsgBuff, 
				   "Cannot remove invalid lock file %s: %s", 
				   LockFile, SYSERR);
		    Error("%s", LockInfo.Error = MsgBuff);
		    return(&LockInfo);
		}
		/* Try again */
	    } else {
		/* Lock is valid or there was an error checking lock */
		(void) sprintf(MsgBuff, "Password file %s is locked by %s", 
			       PwdFile, LockFile);
		Error("%s", LockInfo.Error = MsgBuff);
		return(&LockInfo);
	    }
	} else {
	    (void) sprintf(MsgBuff, "Cannot create lock file %s for %s: %s",
			   LockFile, PwdFile, SYSERR);
	    Error("%s", LockInfo.Error = MsgBuff);
	    return(&LockInfo);
	}
    } while (Count++ == 0);	/* Only loop once */

    (void) sprintf(Buff, "%d\n", getpid());
    BuffLen = strlen(Buff);
    if (write(LockInfo.FileDesc, Buff, BuffLen) != BuffLen) {
	(void) sprintf(MsgBuff, "Write %d bytes to %s failed: %s",
		       BuffLen, LockFile);
	Error("%s", LockInfo.Error = MsgBuff);
	return(&LockInfo);
    }

    return(&LockInfo);
}

/*
 * Unlock password file
 */
void
PWDunlock(LockInfo)
    PWDlock_t		       *LockInfo;
{
    if (LockInfo->FilePtr) {
	(void) fclose(LockInfo->FilePtr);
	LockInfo->FilePtr = NULL;
    }
    if (LockInfo->FileDesc >= 0) {
	(void) close(LockInfo->FileDesc);
	LockInfo->FileDesc = -1;
    }
    if (LockInfo->LockFile) {
	if (unlink(LockInfo->LockFile) != 0)
	    Error("Unlink lock file failed %s: %s", 
		  LockInfo->LockFile, SYSERR);
	LockInfo->LockFile = NULL;
    }
}

/*
 * Parse post op command string.
 */
static char *
PWDgetpostcmd(Facility, PwdFile)
    Facility_t		       *Facility;
    char		       *PwdFile;
{
    static char			Buff[BUFSIZ];
    static char			ErrSmall[] = "Post command buffer too small";
    register char	       *Cmd;
    register char	       *cp;
    register char	       *cp2;

    Buff[0] = CNULL;
    for (Cmd = Facility->PostCmd, cp = Buff; Cmd && *Cmd; ) {
	if (*Cmd == '%') {
	    switch (*++Cmd) {
	    case 'D':	/* YP directory */
		if (strlen(NISdir) + (cp - Buff) + 2 > sizeof(Buff)) {
		    Error(ErrSmall);
		    return((char *) NULL);
		}
		(void) sprintf(cp, "%s", NISdir);
		cp += strlen(NISdir);
		break;
	    case 'B':	/* Base name of passwd file */
		if (cp2 = strrchr(PwdFile, '/'))
		    ++cp2;
		else
		    cp2 = PwdFile;
		if (strlen(cp2) + (cp - Buff) + 2 > sizeof(Buff)) {
		    Error(ErrSmall);
		    return((char *) NULL);
		}
		(void) sprintf(cp, "%s", cp2);
		cp += strlen(cp2);
		break;
	    case '%':
		*cp++ = '%';
		break;
	    default:
		Error("Unknown format character '%c'", *Cmd);
		return((char *) NULL);
	    }
	    ++Cmd;
	} else {
	    if (cp - Buff + 2 > sizeof(Buff)) {
		Error(ErrSmall);
		return((char *) NULL);
	    }
	    *cp++ = *Cmd++;
	}
    }
    *cp = CNULL;

    return( (Buff[0]) ? Buff : (char *) NULL );
}

/*
 * Reap child processes when they exit.
 */
static void Reap(Sig)
    int				Sig;
{
    (void) wait(0);
}

/*
 * Perform post change operations
 */
void
PWDpost(Facility, PwdFile, MsgBuff)
    Facility_t		       *Facility;
    char		       *PwdFile;
    char		       *MsgBuff;
{
    char		       *Command;
    pid_t			ProcID;
    int				Status;

    Command = PWDgetpostcmd(Facility, PwdFile);
    if (!Command)
	return;

    if (BackGround) {
	ProcID = fork();
	if (ProcID < 0) {
	    (void) sprintf(MsgBuff, "Fork for post command failed: %s", 
			   SYSERR);
	    Error("%s", MsgBuff);
	    return;
	} else if (ProcID > 0) {
	    /*
	     * Parent
	     */
	    Message("Spawned pid %d for post command `%s'", ProcID, Command);
	    (void) signal(SIGCHLD, Reap);
	    return;
	}
	/*
	 * Child
	 */
    }

    Message("Running post command `%s'", Command);
    Status = system(Command);
    if (Status == 0)
	Message("Post command completed successfully");
    else
	Error("Post command exited status %d", Status);

    if (BackGround)
	exit(Status);
}

/*
 * See if Arg is a valid field.
 */
char *
PWDvalidfield(Arg, Type)
    char		       *Arg;
    PCtype_t			Type;
{
    static char			ErrBuff[BUFSIZ];
    static char			InvalidArg[] = "Invalid argument";
    static char			InvalidShell[] = "Invalid shell";
    int				MaxSize = 0;
    int				Valid;
    char		       *Shell;
    char		       *cp;

    if (!Arg || *Arg == CNULL) {
	(void) sprintf(ErrBuff, "Argument is empty.");
	return(ErrBuff);
    }

    if (cp = strchr(Arg, ':')) {
	(void) sprintf(ErrBuff, 
		       "Argument contains illegal character `%c' - %*s",
		       *cp, MaxSize, Arg);
	return(ErrBuff);
    }

    switch (Type) {
    case PC_PWD:	MaxSize = MaxPwdSize;		break;
    case PC_GECOS:	MaxSize = MaxGecosSize;		break;
    case PC_SHELL:	MaxSize = MaxShellSize;		break;
    default:
	/* This should only happen if we goof adding a new PC */
	Error("Invalid PC type: %d", Type);
	return(InvalidArg);
    }
    if ((int)strlen(Arg) > MaxSize) {
	(void) sprintf(ErrBuff, "Argument exceeds %d byte limit: %*.s",
		       MaxSize, MaxSize, Arg);
	return(ErrBuff);
    }

    /*
     * See if shell is valid.
     */
    if (Type == PC_SHELL) {
	if (Arg[0] != '/') {
	    (void) sprintf(ErrBuff, "%s: %s", Arg, InvalidShell);
	    return(ErrBuff);
	}

	setusershell();
	Valid = FALSE;
	while (Shell = getusershell())
	    if (EQ(Shell, Arg)) {
		Valid = TRUE;
		break;
	    }
	endusershell();
	if (!Valid) {
	    (void) sprintf(ErrBuff, "%s: %s", Arg, InvalidShell);
	    return(ErrBuff);
	}
    }

    return((char *) NULL);
}

/*
 * Change the Type field of String to NewValue
 */
static char *
PWDchangeLine(String, Type, NewValue)
    char		       *String;
    PCtype_t			Type;
    char		       *NewValue;
{
    struct passwd	       *Passwd;

    Passwd = PWDstrtopwd(String);
    switch (Type) {
    case PC_PWD:	Passwd->pw_passwd = NewValue;	break;
    case PC_GECOS:	Passwd->pw_gecos = NewValue;	break;
    case PC_SHELL:	Passwd->pw_shell = NewValue;	break;
    default:
	/* Should never reach here */
	Error("Unknown password change request: %d", Type);
    }

    return PWDpwdtostr(Passwd);
}

/*
 * Change password entries in a passwd file indicated in Facility.
 *
 * The changes to perform are listed in TransactList.
 * Only password change types (PCtype) indicated by the Type
 * parameter are processed.
 *
 * Each transaction that is successfully completed, needs to be
 * flagged as such.  This allows the transaction file to be deleted
 * later on, if needed.
 */
SPMstat_t
PWDchange(TransactList, Type, Facility, MsgBuff)
    Transact_t		       *TransactList;
    PCtype_t			Type;
    Facility_t		       *Facility;
    char		       *MsgBuff;
{
    static char			Buff[BUFSIZ];
    static char			OldFile[MAXPATHLEN];
    register char	       *cp;
    register Transact_t	       *TransPtr;
    static struct stat		PwdStat;
    char		       *PwdLine;
    char		       *UserName;
    FILE		       *FilePtr;
    FILE		       *TmpFilePtr;
    int				TmpFileDesc;
    char		       *TmpFile;
    char		       *PwdFile;
    char		       *LockFile;
    PWDlock_t		       *LockInfo;
    int				UseShadow = FALSE;
    int				Status;

    if (MsgBuff)
	*MsgBuff = CNULL;

    /*
     * See if there's any transactions of type Type in TransactList.
     */
    if (TRANScheckType(TransactList, Type, MsgBuff) <= 0)
	return(RS_OK);

    /*
     * Decide if we should be updating a shadow password file.
     */
    if (Type == PC_PWD && Facility->ShadowFile && Facility->UpdShadStr &&
	access(Facility->ShadowFile, R_OK) == 0)
	UseShadow = TRUE;

    if (UseShadow) {
	PwdFile = Facility->ShadowFile;
	LockFile = Facility->ShadowLock;
    } else {
	PwdFile = Facility->PwdFile;
	LockFile = Facility->PwdLock;
    }

    if (stat(PwdFile, &PwdStat) != 0) {
	(void) sprintf(MsgBuff, "Cannot stat %s: %s", PwdFile, SYSERR);
	return(RS_ERROR);
    }

    /*
     * Create our tmp working file.
     * To avoid bugs in some file streams implementations, we
     * do an open() of the tmp file, call fchmod(), fchown(), then
     * do an fdopen() to get a FILE pointer.
     */
    TmpFile = PWDgettmp(PwdFile);
    if (!TmpFile) {
	(void) sprintf(MsgBuff, "Cannot make temporary file name");
	return(RS_ERROR);
    }
    TmpFileDesc = open(TmpFile, O_WRONLY|O_CREAT, PwdStat.st_mode);
    if (TmpFileDesc < 0) {
	(void) sprintf(MsgBuff, "Cannot create pwd tmp file %s: %s", 
		       TmpFile, SYSERR);
	return(RS_ERROR);
    }

    /*
     * Set tmp file modes to be the same as the original file.
     */
    if (fchmod(TmpFileDesc, PwdStat.st_mode) != 0) {
	(void) sprintf(MsgBuff, "Cannot chmod file %s: %s", TmpFile, SYSERR);
	(void) close(TmpFileDesc);
	(void) unlink(TmpFile);
	return(RS_ERROR);
    }
    if (fchown(TmpFileDesc, PwdStat.st_uid, PwdStat.st_gid) != 0) {
	(void) sprintf(MsgBuff, "Cannot chown %s to %d.%d: %s",
		       TmpFile, PwdStat.st_uid, PwdStat.st_gid, SYSERR);
	(void) close(TmpFileDesc);
	(void) unlink(TmpFile);
	return(RS_ERROR);
    }

    LockInfo = PWDlock(PwdFile, LockFile, MsgBuff);
    if (!LockInfo || LockInfo->Error) {
	(void) close(TmpFileDesc);
	(void) unlink(TmpFile);
	(void) sprintf(MsgBuff, (LockInfo->Error) ? LockInfo->Error : 
		       "Cannot lock password file");
	return(RS_ERROR);
    }

    FilePtr = fopen(PwdFile, "r");
    if (!FilePtr) {
	(void) sprintf(MsgBuff, "Cannot open for read %s: %s", 
		       PwdFile, SYSERR);
	(void) close(TmpFileDesc);
	(void) unlink(TmpFile);
	PWDunlock(LockInfo);
	return(RS_ERROR);
    }

    TmpFilePtr = fdopen(TmpFileDesc, "w");
    if (!TmpFilePtr) {
	sprintf(MsgBuff, 
		"Cannot fdopen file descriptor (%d) for %s: %s (errno=%d)", 
		TmpFileDesc, TmpFile, SYSERR, errno);
	if (Debug)
	    DescDump(MsgBuff);	/* XXX Debug tracing */
	(void) fclose(FilePtr);
	(void) close(TmpFileDesc);
	(void) unlink(TmpFile);
	PWDunlock(LockInfo);
	if (ExitOnError) {
	    Error("%s", MsgBuff);
	    exit(2);
	} else {
	    return(RS_ERROR);
	}
    }

    while (fgets(Buff, sizeof(Buff), FilePtr)) {
	cp = strchr(Buff, ':');
	if (cp) {
	    *cp = CNULL;			/* Zap at first field */
	    UserName = Buff;
	} else {
	    UserName = NULL;
	}

	if (UserName && (TransPtr = TRANSgetByUser(UserName, TransactList, 
						   Type))) {
	    /*
	     * Modify current line
	     */
	    *cp = ':';				/* Put back field seperator */
	    if (UseShadow)
		PwdLine = (*Facility->UpdShadStr)(Buff, TransPtr->NewValue);
	    else
		PwdLine = PWDchangeLine(Buff, Type, TransPtr->NewValue);

	    if (!PwdLine) {
		(void) fclose(FilePtr);
		(void) fclose(TmpFilePtr);
		(void) close(TmpFileDesc);
		(void) unlink(TmpFile);
		PWDunlock(LockInfo);
		(void) sprintf(MsgBuff, "Passwd entry conversion failed");
		return(RS_ERROR);
	    }
	    /* Mark it TODO */
	    TransPtr->Flags |= TF_TODO;
	} else {
	    /*
	     * Use line from original file.
	     */
	    if (cp) *cp = ':';			/* Put back field seperator */
	    PwdLine = Buff;
	}

	if (fputs(PwdLine, TmpFilePtr) == EOF) {
	    (void) sprintf(MsgBuff, "Write failed: %s: %s", TmpFile, SYSERR);
	    (void) fclose(FilePtr);
	    (void) fclose(TmpFilePtr);
	    (void) close(TmpFileDesc);
	    (void) unlink(TmpFile);
	    PWDunlock(LockInfo);
	    return(RS_ERROR);
	}
    }

    /*
     * Close file descriptors
     */
    if (fclose(TmpFilePtr) == EOF) {
	(void) sprintf(MsgBuff, "fclose failed %s: %s", TmpFile, SYSERR);
	(void) fclose(FilePtr);
	(void) close(TmpFileDesc);
	(void) unlink(TmpFile);
	PWDunlock(LockInfo);
	return(RS_ERROR);
    }
    /* 
     * XXX TmpFileDesc should already be closed (by the 
     * above fclose(TmpFilePtr), but we want to be
     * paranoid about file leaks.
     */
    (void) close(TmpFileDesc);		
    (void) fclose(FilePtr);
    LockInfo->FilePtr = FilePtr = NULL;

    /*
     * Save the current passwd file
     */
    (void) sprintf(OldFile, "%s.OLD", PwdFile);
    (void) unlink(OldFile);
    if (rename(PwdFile, OldFile) != 0) {
	(void) sprintf(MsgBuff, "Rename of %s to %s failed: %s",
		       PwdFile, OldFile, SYSERR);
	(void) unlink(TmpFile);
	PWDunlock(LockInfo);
	return(RS_ERROR);
    }

    /*
     * Install the new file
     */
    if (rename(TmpFile, PwdFile) != 0) {
	(void) sprintf(MsgBuff, "Rename of %s to %s failed: %s",
		       TmpFile, PwdFile, SYSERR);
	(void) unlink(TmpFile);
	if (rename(OldFile, PwdFile) == 0)
	    Error("Re-installed %s from %s", PwdFile, OldFile);
	else
	    Error("Could not re-install %s to %s: %s", 
		  OldFile, PwdFile, SYSERR);
	PWDunlock(LockInfo);
	return(RS_ERROR);
    }

    /*
     * Mark all the transactions we've done
     */
    TRANSmark(TransactList, Type);

    /*
     * Success!
     */
    PWDunlock(LockInfo);
    PWDpost(Facility, PwdFile, MsgBuff);

    return(RS_OK);
}

/*
 * Parse a GECOS string and place values into OldVal in
 * each element of the GecosList linked list.
 */
Gecos_t *
PWDgetgecos(String, GecosList)
    char		       *String;
    Gecos_t		       *GecosList;
{
    static char		       *Buffer;
    register Gecos_t	       *Gecos;
    register char	       *Loc;
    register char	       *End;
    register char	       *cp;

    if (Buffer)
	(void) free(Buffer);

    Loc = Buffer = strdup(String);
    End = &Buffer[strlen(Buffer)-1];
    for (cp = strchr(Loc, *GecosSep), Gecos = GecosList; cp < End && Gecos; 
	 cp = strchr(Loc, *GecosSep), Gecos = Gecos->Next) {
	if (cp)
	    *cp = CNULL;
	if (Loc && *Loc)
	    Gecos->OldVal = Loc;
	if (cp)
	    Loc = cp+1;
    }

    return(GecosList);
}
