/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */

/*
 * $Header: /src/common/cmd/spm/2.x/RCS/spm.x,v 1.18 1996/03/02 00:43:13 mcooper Exp $
 *
 * System for Password Management RPC description.
 */

/*
 * General type definetions
 */
typedef string string_t<>;		/* Arbitrary string */

/*
 * SPM basic query information.
 */
struct SPMquery {
	string_t	UserName;	/* User name */
	string_t	ReqUser;	/* User sending request */
	string_t	Facility;	/* Name of facility to use */
	string_t	Domain;		/* Domain to check */
	string_t	CurPass;	/* Current password */
	string_t	Arg;		/* Argument */
};
typedef struct SPMquery SPMquery_t;

/*
 * SPM call status values.
 */
enum SPMstat {
	RS_OK = 1,			/* Operation success */
	RS_ERROR,			/* Failed: See error string */
	RS_NOENT,			/* Failed: No such entry */
	RS_MATCH,			/* Success: Found match */
	RS_NOMATCH			/* No match */
};
typedef struct SPMstat SPMstat_t;

/*
 * Result of a SPM query.
 */
struct SPMresult {
	enum SPMstat	Status;		/* Status of query */
	string_t	Message;	/* Message string */
};
typedef struct SPMresult SPMresult_t;

/* 
 * Password Aging Data 
 */
struct AGEinfo {	
	string_t	UserName;	/* User name */
	time_t		ModTime;	/* Password last modified time */
	int		MinChg;		/* Minimum # of days between changes */
	int		MaxValid;	/* Maximum # of days pwd is valid */
	int		WarnTime;	/* Number of days of warnings */
	int		Inactive;	/* # of days inactive b4 disable */
	time_t		ExpireDate;	/* Absolute date of acct expiration */
	int		Flags;		/* Flags */
	SPMresult	Result;		/* Results of SPM query */
};
typedef struct AGEinfo AGEinfo_t;

/* 
 * Personel Data - Version 1
 */
struct PDinfo {
	string_t	UserName;	/* User name */
	string_t	ReqUser;	/* User making request */
	string_t	Domain;		/* Domain name */
	string_t	CurPass;	/* Current password */
	string_t	FacName;	/* Name of pwd facility */
	time_t		CrtTime;	/* Time entry was created */
	time_t		ModTime;	/* Time entry was last modified */
	string_t	SSN;		/* Social Security Number */
	string_t	DOB;		/* Date Of Birth */
	string_t	MomName;	/* Mother's Maiden Name */
	string_t	HomeZip;	/* Home ZIP code */
	SPMresult	Result;		/* Results of SPM query */
};
typedef struct PDinfo PDinfo_t;

/*
 * Personel Data item
 */
struct PDitem {
	string_t	Key;		/* Keyword for item */
	string_t	Desc;		/* Description */
	string_t	Example;	/* Example input */
	long		Flags;		/* Flags */
	string_t	Value;		/* Input value */
	struct PDitem  *Next;		/* Ptr to next member */
};
typedef struct PDitem	PDitem_t;

/* 
 * Personel Data - Version 2
 */
struct PDinfo2 {
	string_t	UserName;	/* User name */
	string_t	ReqUser;	/* User making request */
	string_t	Domain;		/* Domain name */
	string_t	CurPass;	/* Current password */
	string_t	FacName;	/* Name of pwd facility */
	time_t		CrtTime;	/* Time entry was created */
	time_t		ModTime;	/* Time entry was last modified */
	PDitem	       *Items;		/* Item list */
	SPMresult	Result;		/* Results of SPM query */
};
typedef struct PDinfo2 PDinfo2_t;

program SPMPROG {
	version SPMVERS_1 {
		/* 
		 * Set password
		 */
		SPMresult
		SPMPROC_SETPWD(SPMquery) = 1;

		/* 
		 * Set shell
		 */
		SPMresult
		SPMPROC_SETSHELL(SPMquery) = 2;

		/* 
		 * Set GECOS
		 */
		SPMresult
		SPMPROC_SETGECOS(SPMquery) = 3;

		/*
		 * Check password history
		 */
		SPMresult
		SPMPROC_CHECKPWDHIST(SPMquery) = 4;

		/*
		 * Get personel data information
		 */
		PDinfo
		SPMPROC_GETPD(SPMquery) = 5;

		/*
		 * Check personel data information
		 */
		SPMresult
		SPMPROC_CHECKPD(PDinfo) = 6;

		/*
		 * Check password
		 */
		SPMresult
		SPMPROC_CHECKPWD(SPMquery) = 7;
	} = 1;
	version SPMVERS_2 {
		/* 
		 * Set password
		 */
		SPMresult
		SPMPROC_SETPWD(SPMquery) = 1;

		/* 
		 * Set shell
		 */
		SPMresult
		SPMPROC_SETSHELL(SPMquery) = 2;

		/* 
		 * Set GECOS
		 */
		SPMresult
		SPMPROC_SETGECOS(SPMquery) = 3;

		/*
		 * Check password history
		 */
		SPMresult
		SPMPROC_CHECKPWDHIST(SPMquery) = 4;

		/*
		 * Get personel data information
		 */
		PDinfo2
		SPMPROC_GETPD(SPMquery) = 5;

		/*
		 * Check personel data information
		 */
		SPMresult
		SPMPROC_CHECKPD(PDinfo2) = 6;

		/*
		 * Check password
		 */
		SPMresult
		SPMPROC_CHECKPWD(SPMquery) = 7;
	} = 2;
} = 232323;
