/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/spmd.c,v 1.32 1996/03/02 00:43:43 mcooper Exp $";
#endif	/* lint */

/*
 * System for Password Management server
 */

#include "defs.h"
#include "options.h"
#include <signal.h>
#include <sys/resource.h>

#if 	defined(sunos5)
#include <rpc/svc_soc.h>
#endif	/* sunos5 */

char		       *ConfigFile = SPMD_CONF;	/* Configuration file */
char		      **MainArgv = NULL;
int			MainArgc = -1;
int			DoVersion = 0;		/* Show version info */

/* 
 * For rpcgen/dispatch.c 
 * At least required for SunOS 5.5
 */
int			_rpcpmstart = 1;

/*
 * Command line options table.
 */
OptionDescRec Opts[] = {
    {"-agedir",	SepArg,		OptStr, 	__ &AgeDir,		NULL,
	 "<name>",	"Name of Age Directory"},
    {"-bg",	NoArg,		OptBool, 	__ &BackGround,		"0",
	 NULL,	"Do not background certain tasks"},
    {"+bg",	NoArg,		OptBool, 	__ &BackGround,		"1",
	 NULL,	"Background certain tasks"},
    {"-pddir",	SepArg,		OptStr, 	__ &PDdir,		NULL,
	 "<name>",	"Name of Personal Data Directory"},
    {"-pwdhistdir",SepArg,	OptStr, 	__ &PWHdir,		NULL,
	 "<name>",	"Name of Passwd History Directory"},
    {"-nispostcmd",SepArg,	OptStr, 	__ &NISPostCmd,		NULL,
	 "<command>",	"Command to run after NIS update"},
    {"-nispwdfile",SepArg,	OptStr, 	__ &NISPwdFile,		NULL,
	 "<name>",	"Name of NIS password file"},
    {"-nispwdlock",SepArg,	OptStr, 	__ &NISPwdLock,		NULL,
	 "<name>",	"Name of NIS password lock file"},
    {"-localpwdfile",SepArg,	OptStr, 	__ &LocalPwdFile,	NULL,
	 "<name>",	"Name of local password file"},
    {"-localpwdlock",SepArg,	OptStr, 	__ &LocalPwdLock,	NULL,
	 "<name>",	"Name of local password lock file"},
    {"-localpostcmd",SepArg,	OptStr, 	__ &LocalPostCmd,	NULL,
	 "<command>",	"Command to run after local update"},
    {"-transdir",	SepArg,	OptStr, 	__ &TransDir,		NULL,
	 "<name>",	"Name of Transactions Directory"},
    {"-transfaclist",	SepArg,	OptStr, 	__ &TransFacList,	NULL,
	 "<Facility List>","Names of facilities to do transactions on"},
    {"-version",NoArg,		OptBool, 	__ &DoVersion,		"1",
	 NULL,	"Show version information"},
    {"-debug",ArgHidden|SepArg,	OptInt, 	__ &Debug,		"1",
	 NULL,	"Enable debugging"},
};

/*
 * Set configuration information.
 */
static int
SetConfig()
{
    CONFinit();

    if (CONFparse(ConfigFile, VarTable, NULL) != 0)
	return(-1);

    if (ParseOptions(Opts, Num_Opts(Opts), MainArgc, MainArgv) < 0)
	return(-1);

    if (DoVersion) {
	printf("%s\n", GetVersion());
	exit(0);
    }

    CONFinstall();

    return(0);
}

/*
 * Signal handler
 */
static void
SigHandler(sig)
    int				sig;
{
    switch (sig) {
    case SIGHUP:		/* Re-read config info */
	(void) SetConfig();
	break;
    }
}

static int
CheckDirs()
{
    struct stat			StatBuf;

#if	defined(OLD_DIR)
    /*
     * See if our old directory is out there and fix things up if needed.
     */
    if (stat(TOP_DIR, &StatBuf) < 0 && stat(OLD_DIR, &StatBuf) == 0) {
	if (rename(OLD_DIR, TOP_DIR) != 0)
	    Error("Cannot rename %s -> %s: %s.", OLD_DIR, TOP_DIR, SYSERR);
	else
	    Message("Renamed `%s' to `%s'", OLD_DIR, TOP_DIR);
    }
#endif	/* OLD_DIR */

    if (CheckDir(AgeDir) < 0)
	return(-1);
    if (CheckDir(PDdir) < 0)
	return(-1);
    if (CheckDir(PWHdir) < 0)
	return(-1);

    return(0);
}

static int
Init(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    FILE		       *FilePtr;
    char		       *Version;
    char		       *cp;
#if	defined(RLIMIT_NOFILE)
    struct rlimit		rlimit;

    /*
     * Increase max number of open files to max to try and avoid
     * bugs in SunOS 4.
     */
    if (getrlimit(RLIMIT_NOFILE, &rlimit) == 0) {
	rlimit.rlim_cur = rlimit.rlim_max;
	if (setrlimit(RLIMIT_NOFILE, &rlimit) == -1)
	    Error("setrlimit(RLIMIT_NOFILE) to %d failed: %s",
		  rlimit.rlim_cur, SYSERR);
    } else {
	Error("getrlimit(RLIMIT_NOFILE) failed: %s", SYSERR);
    }
#endif	/* RLIMIT_NOFILE */

    (void) umask(UMASK_MODE);

    /*
     * Assume we're being run from inetd and that it's okay to
     * exit on error (because inetd will restart a new process).
     */
    ExitOnError = 1;

    ProgramName = Basename(Argv[0], '/');	/* Our program name */
    UseSyslog = 1;				/* Log to syslog()? */

    (void) openlog(ProgramName, LOG_PID, LOG_AUTH);
    if (Version = GetVersion())
	if (cp = strchr(Version, '\n'))
	    *cp = CNULL;
    Message("starting %s", (Version) ? Version : "");

    MainArgv = Argv;
    MainArgc = Argc;
    if (SetConfig() != 0)
	exit(1);

    (void) signal(SIGHUP, SigHandler);

    /*
     * Write our pid to the PID file
     */
    FilePtr = fopen(SPMD_PID, "w");
    if (FilePtr) {
	(void) fchmod(fileno(FilePtr), FILE_MODE);
	(void) fprintf(FilePtr, "%d\n", getpid());
	(void) fclose(FilePtr);
    } else
	Error("Cannot create `%s': %s.", SPMD_PID, SYSERR);

    return(0);
}

main(Argc, Argv)
    int				Argc;
    char		      **Argv;
{
    SVCXPRT		       *Transport;
    int				Sock = RPC_ANYSOCK;
    void			spmprog_1();
    void			spmprog_2();

    /*
     * Need initial setting.
     */
    ProgramName = Argv[0];

    if (getuid() != 0) {
	Error("This program must be run as ``root''.");
	exit(1);
    }

    if (Init(Argc, Argv) < 0)
	exit(1);

    if (CheckDirs() < 0)
	exit(1);

#ifdef notdef
    /* svcudp_create() normally binds to a reserved port */
    if ((Sock = GetResSock()) < 0) {
	Error("Cannot get reserved port.");
	exit(1);
    }
#endif	/* notdef */

#if	TRANS_TYPE == TRANS_UDP
    Transport = svcudp_create(Sock);
#endif	/* TRANS_UDP */
#if	TRANS_TYPE == TRANS_TCP
    Transport = svctcp_create(Sock, 0, 0);
#endif	/* TRANS_TCP */
    if (!Transport) {
	Error("Could not create RPC service.");
	exit(1);
    }

    (void) pmap_unset(SPMPROG, SPMVERS_1);
    (void) pmap_unset(SPMPROG, SPMVERS_2);

    if (!svc_register(Transport, SPMPROG, SPMVERS_1, spmprog_1, 
#if	TRANS_TYPE == TRANS_UDP
		      IPPROTO_UDP
#endif	/* TRANS_UDP */
#if	TRANS_TYPE == TRANS_TCP
		      IPPROTO_TCP
#endif	/* TRANS_UDP */
		      )) {
	Error("Could not register RPC service (Version %d).", SPMVERS_1);
	exit(1);
    }
    if (!svc_register(Transport, SPMPROG, SPMVERS_2, spmprog_2, 
#if	TRANS_TYPE == TRANS_UDP
		      IPPROTO_UDP
#endif	/* TRANS_UDP */
#if	TRANS_TYPE == TRANS_TCP
		      IPPROTO_TCP
#endif	/* TRANS_UDP */
		      )) {
	Error("Could not register RPC service (Version %d).", SPMVERS_2);
	exit(1);
    }

    Message("ready for service");
    svc_run();		/* Should never return */
    Error("Service run returned unexpectedly.");
    exit(1);
}
