/*
 * Copyright 1995-1996, Michael A. Cooper
 * This software may be freely distributed provided that the Author is fully
 * credited, and the software is not sold for profit or used in any for-profit
 * product without the express written consent of the Author.
 */
#ifndef lint
static char RCSid[] = 
"$Header: /src/common/cmd/spm/2.x/RCS/trans.c,v 1.24 1996/03/02 00:44:35 mcooper Exp $";
#endif	/* lint */

/*
 * Transaction functions
 */

#include "defs.h"

/*
 * Is transaction logging enabled for FacName?
 */
int
TRANSenabled(FacName)
    char		       *FacName;
{
    int				Match = 0;
    static char		      **Argv = NULL;
    static int			Argc = 0;
    register int		i;

    if (!TransFacList)
	return(0);

    Argc = StrToArgv(TransFacList, ",", &Argv, SF_RMWS);
    for (i = 0; i < Argc; ++i) {
	if (strcmp(FacName, Argv[i]) == 0) {
	    Match = 1;
	    break;
	}
    }

    DestroyArgv(&Argv, Argc);

    return(Match);
}

/*
 * Get transaction ID from the transaction file name.
 */
char *
TRANSgetIdent(File)
    char		       *File;
{
    register char	       *Base;
    register char	       *cp;

    Base = strrchr(File, '/');
    if (Base)
	++Base;
    else
	Base = File;

    /* skip over TRANS_PREFIX */
    cp = strchr(Base, '.');
    if (cp)
	++cp;
    else
	cp = Base;

    return(cp);
}

/*
 * Get the transaction directory name for Facility.
 */
char *
TRANSgetDir(Facility)
    Facility_t		       *Facility;
{
    static char			Path[MAXPATHLEN];

    (void) sprintf(Path, "%s/%s", TransDir, Facility->Name);

    return(Path);
}

/*
 * Find the transaction in TransList for UserName with PCtype of Type.
 */
Transact_t *
TRANSgetByUser(UserName, TransactList, Type)
    char		       *UserName;
    Transact_t		       *TransactList;
    PCtype_t			Type;
{
    register Transact_t	       *TransPtr;

    for (TransPtr = TransactList; TransPtr; TransPtr = TransPtr->Next) {
	if (IS_ON(TransPtr->Flags, TF_IGNORE))
	    continue;
	/* See if we've already marked this transaction for processing */
	if (IS_ON(TransPtr->Flags, TF_TODO))
	    continue;
	if (TransPtr->Type == Type && EQ(TransPtr->UserName, UserName))
	    return(TransPtr);
    }

    return((Transact_t *) NULL);
}

/*
 * Make the transaction file
 */
static int
TRANSmakeFile(Facility, TransFile, TransFileSize, ErrBuff)
    Facility_t		       *Facility;
    char		       *TransFile;
    size_t			TransFileSize;
    char		       *ErrBuff;
{
    int				Desc = 0;
    char		       *DirName;
    static char		        File[MAXPATHLEN];
    time_t			TimeVal;
    size_t			DirLen;
    register long		Counter;

    DirName = TRANSgetDir(Facility);
    if (!DirName) {
	(void) sprintf(ErrBuff, "Cannot get transaction directory name");
	return(-1);
    }
    if (CheckDir(DirName) == -1) {
	(void) sprintf(ErrBuff, "Check directory for \"%s\" failed.", 
		       DirName);
	return(-1);
    }
    DirLen = (size_t) strlen(DirName);

    /*
     * Create a unique filename based on the current time and
     * an incremented counter.
     */
    for (Counter = 0; Counter < TRANS_MAX_CREATE_TRIES; ++Counter) {
	(void) time(&TimeVal);
	(void) sprintf(File, "%s.%d.%d", TRANS_PREFIX, TimeVal, Counter);
	if (strlen(File) + DirLen + 2 >= TransFileSize) {
	    (void) sprintf(ErrBuff, "TRANSmakeFile() Buffer too small!");
	    return(-1);
	}
	(void) sprintf(TransFile, "%s/%s", DirName, File);

	Desc = open(TransFile, O_WRONLY|O_CREAT|O_EXCL, FILE_MODE);
	if (Desc == -1) {
	    if (errno == EEXIST)
		/* The file exists, so try again */
		continue;
	    (void) sprintf(ErrBuff, "Create transaction file failed: %s: %s", 
			   TransFile, SYSERR);
	    return(-1);
	} else {
	    return(Desc);
	}
    }

    (void) sprintf(ErrBuff, "Could not create a unique transaction file.");
    return(-1);
}

/*
 * Add a transaction.
 * Return a message on error or NULL on success.
 */
SPMstat_t
TRANSadd(Transact, Facility, MsgBuff)
    Transact_t		       *Transact;
    Facility_t		       *Facility;
    char		       *MsgBuff;
    /*ARGSUSED*/
{
    static char			TransBuff[BUFSIZ * 2];
    static char			TransFile[MAXPATHLEN];
    size_t			TransLen;
    int				TransDesc = 0;
    char		       *TransType;
    char		       *OldValue;
    char		       *cp;

    /*
     * Validate argument.
     */
    if (cp = PWDvalidfield(Transact->NewValue, Transact->Type)) {
	if (MsgBuff) (void) strcpy(MsgBuff, cp);
	return(RS_ERROR);
    }

    /*
     * Find the current value of what we're about to change
     */
    OldValue = FACgetByType(Transact->UserName, Transact->Type, Facility);
    if (!OldValue) {
	if (MsgBuff) 
	    (void) sprintf(MsgBuff, 
			   "Cannot find current value for \"%s\" for \"%s\"",
			   GetPCname(Transact->Type), Transact->UserName);
	return(RS_ERROR);
    }

    /*
     * Make the transaction file
     */
    TransDesc = TRANSmakeFile(Facility, TransFile, sizeof(TransFile), MsgBuff);
    if (TransDesc < 0)
	return(RS_ERROR);

    /*
     * Lock the file
     */
    if (FILElock(TransDesc, F_WRLCK, TransFile, MsgBuff) != 0) {
	(void) close(TransDesc);
	(void) unlink(TransFile);
	return(RS_ERROR);
    }

    switch (Transact->Type) {
    case PC_PWD:	TransType = PCS_PWD;		break;
    case PC_SHELL:	TransType = PCS_SHELL;		break;
    case PC_GECOS:	TransType = PCS_GECOS;		break;
    default:
	if (MsgBuff)
	    (void) sprintf(MsgBuff, 
			   "Unknown PC Type request for transactions: %d",
			   Transact->Type);
	(void) close(TransDesc);
	(void) unlink(TransFile);
	return(RS_ERROR);
    }

    if ((strlen(Transact->UserName) + 1 + strlen(TransType) + 1 + 
	 strlen(OldValue) + 1 +
	 strlen(Transact->NewValue) + 2) >= sizeof(TransBuff)) {
	if (MsgBuff) (void) sprintf(MsgBuff, "Transaction buffer too small.");
	(void) close(TransDesc);
	(void) unlink(TransFile);
	return(RS_ERROR);
    }

    /*
     * Format transaction
     */
    (void) sprintf(TransBuff, "%s%s%s%s%s%s%s\n",
		   Transact->UserName, TRANS_FLD_SEP,
		   TransType, TRANS_FLD_SEP,
		   OldValue, TRANS_FLD_SEP,
		   Transact->NewValue);
    TransLen = strlen(TransBuff);

    /*
     * Write the transaction to the file
     */
    if (write(TransDesc, TransBuff, TransLen) != TransLen) {
	if (MsgBuff) (void) sprintf(MsgBuff, 
				    "Write to transaction file failed: %s: %s",
				    TransFile, SYSERR);
	(void) close(TransDesc);
	(void) unlink(TransFile);
	return(RS_ERROR);
    }

    if (close(TransDesc) == -1) {
	if (MsgBuff) (void) sprintf(MsgBuff, 
				    "Close of transaction file failed: %s: %s",
				    TransFile, SYSERR);
	(void) close(TransDesc);
	(void) unlink(TransFile);
	return(RS_ERROR);
    }

    (void) FILEunlock(TransDesc, TransFile, MsgBuff);

    /*
     * Success!
     */
    Message("trans add %s %.*s", TRANSgetIdent(TransFile), 
	    TransLen-1, TransBuff);
    if (MsgBuff)
	(void) sprintf(MsgBuff, "Your request to change your %s\ninformation has been queued for later processing.",
		       GetPCname(Transact->Type));

    return(RS_OK);
}

/*
 * Delete transaction Transact
 */
int
TRANSdelete(Transact)
    Transact_t		       *Transact;
{
    int				Status = 0;

    if (Transact->File)
	if (unlink(Transact->File) != 0) {
	    Error("Delete trans file failed %s: %s", Transact->File, SYSERR);
	    --Status;
	}

    if (Transact->Desc > 0) {
	(void) FILEunlock(Transact->Desc, Transact->File, NULL);
	(void) close(Transact->Desc);
    }

    return(Status);
}

/*
 * Parse a transaction string and return a Transact_t pointer.
 */
Transact_t *
TRANSgetTrans(String, File, Desc, ErrBuff)
    char		       *String;
    char		       *File;
    int				Desc;
    char		       *ErrBuff;
{
    register char	       *cp;
    register int		i;
    Transact_t		       *NewTrans = NULL;
    PCtype_t			Type;
    static char		      **Argv = NULL;
    static int			Argc = 0;

    if (Argv && Argc)
	DestroyArgv(&Argv, Argc);

    if (cp = strchr(String, '\n'))
	*cp = CNULL;

    Argc = StrToArgv(String, TRANS_FLD_SEP, &Argv, SF_RMWS);
    if (!Argc) {
	(void) sprintf(ErrBuff, 
		       "Unable to parse transaction buffer \"%s\".",
		       String);
	return((Transact_t *) NULL);
    }

    if (Argc != TRANS_FLD_CNT) {
	(void) sprintf(ErrBuff, 
  "Wrong number of fields in transaction string \"%s\" (found %d, wanted %d)",
		       String, Argc, TRANS_FLD_CNT);
	return((Transact_t *) NULL);
    }

    /*
     * Check for empty fields.
     * Only the OldValue field may be empty.
     */
    if (!Argv[0]) {
	(void) sprintf(ErrBuff, "Empty UserName field in %s", String);
	return((Transact_t *) NULL);
    }
    if (!Argv[1]) {
	(void) sprintf(ErrBuff, "Empty ChangeType field in %s", String);
	return((Transact_t *) NULL);
    }
    if (!Argv[3]) {
	(void) sprintf(ErrBuff, "Empty NewValue field in %s", String);
	return((Transact_t *) NULL);
    }

    Type = GetPCtype(Argv[1]);
    if (Type == PC_UNKNOWN) {
	(void) sprintf(ErrBuff, "Unknown PC type string \"%s\".", Argv[1]);
	return((Transact_t *) NULL);
    }

    NewTrans = (Transact_t *) xcalloc(1, sizeof(Transact_t));
    NewTrans->File = strdup(File);
    NewTrans->Desc = Desc;

    NewTrans->UserName = strdup(Argv[0]);
    NewTrans->Type = Type;
    if (Argv[2])	/* It's OK for the old value to be NULL */
	NewTrans->OldValue = strdup(Argv[2]);
    else
	NewTrans->OldValue = "";
    NewTrans->NewValue = strdup(Argv[3]);
    NewTrans->Ident = strdup(TRANSgetIdent(File));

    return(NewTrans);
}

/*
 * Check a transaction to see if it's okay to process it.
 *
 * Return -1 on error.
 * Return 0 if it's okay.
 * Return 1 when request is invalid (out of date)
 */
int
TRANScheck(Transact, Facility, ErrBuff)
    Transact_t		       *Transact;
    Facility_t		       *Facility;
    char		       *ErrBuff;
{
    char		       *What = NULL;
    char		       *TypeStr = NULL;
    char		       *Current;
    SPMstat_t			Status;

    TypeStr = GetPCname(Transact->Type);
    /*
     * Make sure that what's in the password (or shadow) database currently
     * matches the old value that's stored in the transaction.  If the current
     * database value is different, then we discard the transaction as being
     * out of date.
     */
    Current = FACgetByType(Transact->UserName, Transact->Type, Facility);
    if (!Current) {
	(void) sprintf(ErrBuff, "Cannot find current %s info for %s",
		       TypeStr, Transact->UserName);
	return(1);
    }
    if (!EQ(Current, Transact->OldValue)) {
	(void) sprintf(ErrBuff, 
		       "The %s for %s has changed",
		       TypeStr, Transact->UserName);
	return(1);
    }

    return(0);
}

/*
 * Return which of Trans1 and Trans2 is newest.
 */
static Transact_t *
TRANSgetNewest(Trans1, Trans2)
    Transact_t		       *Trans1;
    Transact_t		       *Trans2;
{
    int				Val;

    if (!Trans1->Ident || !Trans2->Ident) {
	Error("TRANSgetNewest(): NULL Ident value passed.");
	return((Transact_t *) NULL);
    }

    Val = strcmp(Trans1->Ident, Trans2->Ident);
    if (Val == 0)
	/* They are the same! */
	return(Trans1);
    else if (Val > 0) 
	return(Trans1);
    else
	return(Trans2);
}

/*
 * Insert Transaction NewTrans into TransList
 */
void
TRANSinsert(NewTrans, TransList)
    Transact_t		       *NewTrans;
    Transact_t		      **TransList;
{
    register Transact_t	       *TransPtr;
    register Transact_t	       *DiscardPtr;
    Transact_t		       *NewestTrans = NULL;

    /*
     * See if a transaction for the same UserName + Change Type
     * that's in NewTrans exists in TransList.
     */
    for (TransPtr = *TransList; TransPtr; TransPtr = TransPtr->Next) {
	if (!(EQ(TransPtr->UserName, NewTrans->UserName) && 
	      TransPtr->Type == NewTrans->Type))
	    continue;
	/* Figure out which is the newest transaction */
	NewestTrans = TRANSgetNewest(NewTrans, TransPtr);
	if (NewestTrans == NewTrans) {
	    /* Replace TransPtr with NewTrans */
	    NewTrans->Next = TransPtr->Next;
	    NewTrans->Prev = TransPtr->Prev;
	    if (TransPtr->Next)
		TransPtr->Next->Prev = NewTrans;	/* Child */
	    if (TransPtr->Prev)
		TransPtr->Prev->Next = NewTrans;	/* Parent */
	    /* Add transact we're replacing to our discard list */
	    NewTrans->Discards = TransPtr;
	    /* If TransPtr is the start of TransList, update pointer */
	    if (*TransList == TransPtr)
		*TransList = NewTrans;
	} else if (NewestTrans == TransPtr) {
	    /*
	     * The new transaction is older than the current list item
	     * so we add it to the current item's discard list.
	     */
	    if (TransPtr->Discards) {
		for (DiscardPtr = TransPtr->Discards; 
		     DiscardPtr && DiscardPtr->Next; 
		     DiscardPtr = DiscardPtr->Next);
		DiscardPtr->Next = NewTrans;
	    } else
		TransPtr->Discards = NewTrans;
	}
	return;
    }

    /*
     * If we made it this far, then we did not insert NewTrans
     * into TransList so we add it now.
     */
    if (*TransList) {
	/* Append to end of TransList */
	for (TransPtr = *TransList; TransPtr && TransPtr->Next; 
	     TransPtr = TransPtr->Next);
	TransPtr->Next = NewTrans;
	NewTrans->Prev = TransPtr;
    } else
	*TransList = NewTrans;
}

/*
 * Read the transaction in File into memory and return a Transact_t for it.
 */
Transact_t *
TRANSread(File, Facility, ErrBuff)
    char		       *File;
    Facility_t		       *Facility;
    char		       *ErrBuff;
{
    int				Desc;
    static char			Buff[BUFSIZ];
    size_t			Amt;

    Desc = open(File, O_RDONLY);
    if (Desc == -1) {
	(void) sprintf(ErrBuff, "Open transaction file failed: %s: %s", 
		       File, SYSERR);
	return((Transact_t *) NULL);
    }

    /*
     * Lock the file
     */
    if (FILElock(Desc, F_RDLCK, File, ErrBuff) != 0) {
	(void) close(Desc);
	return((Transact_t *) NULL);
    }

    /*
     * Read the file
     */
    Amt = read(Desc, Buff, sizeof(Buff));
    if (Amt == -1) {
	(void) sprintf(ErrBuff, "Error reading transaction file %s: %s",
		       File, SYSERR);
    } else if (Amt == 0) {
	(void) sprintf(ErrBuff, "Transaction file is empty: %s", File);
    } else {
	/*
	 * Get and place data into a Transact_t
	 */
	return(TRANSgetTrans(Buff, File, Desc, ErrBuff));
    }

    return((Transact_t *) NULL);
}

/*
 * When processing transactions, should we skip this transaction?
 */
int
TRANSskip(Transact, Type)
    Transact_t		       *Transact;
    PCtype_t			Type;
{    
    if (IS_ON(Transact->Flags, TF_IGNORE))
	return(TRUE);
    if (Transact->Type != Type)
	return(TRUE);

    return(FALSE);
}

/*
 * Look through TransactList and return the number of valid entries
 * of type Type that are found.
 */
int
TRANScheckType(TransactList, Type, MsgBuff)
    Transact_t		       *TransactList;
    PCtype_t			Type;
    char		       *MsgBuff;
{
    register Transact_t	       *TransPtr;
    register int		Status;
    register char	       *cp;

    /*
     * Scan Transaction list to see if there are any obvious errors
     * and to make sure there is actually something to do.
     */
    for (TransPtr = TransactList, Status = 0; TransPtr; 
	 TransPtr = TransPtr->Next) {
	if (TRANSskip(TransPtr, Type))
	    continue;
	/*
	 * Validate argument.
	 */
	if (cp = PWDvalidfield(TransPtr->NewValue, Type)) {
	    TransPtr->Result.Status = RS_ERROR;
	    TransPtr->Result.Message = strdup(cp);
	} else
	    ++Status;
    }
    if (Status == 0) {
	if (MsgBuff)
	    (void) sprintf(MsgBuff, 
		  "There are no valid %s change transactions to perform.",
			   GetPCname(Type));
	return(RS_OK);
    }

    return(Status);
}

/*
 * Mark all transactions in TransactList as TF_DONE that are of type Type
 * and marked as TF_TODO.
 */
void
TRANSmark(TransactList, Type)
    Transact_t		       *TransactList;
    PCtype_t			Type;
{
    register Transact_t	       *TransPtr;

    /*
     * Update status of all transactions we've done.
     */
    for (TransPtr = TransactList; TransPtr; TransPtr = TransPtr->Next)
	if (TransPtr->Type == Type && IS_ON(TransPtr->Flags, TF_TODO)) {
	    TransPtr->Flags |= TF_DONE;
	    TransPtr->Result.Status = RS_OK;
	}
}
